@@ -35,10 +35,6 @@ typedef struct {
 	double x, y;
 } Point;
 
-typedef struct {
-	int start, cur;
-} TextSelBounds;
-
 typedef enum {
 	AnchorNorthWest, AnchorNorth, AnchorNorthEast,
 	AnchorLineWest, AnchorLine, AnchorLineEast,
