@@ -23,12 +23,16 @@
  */
 
 #include "config.h"
+#include "group.h"
 #include "text.h"
+#include "text-client.h"
 #include "text-tag.h"
 #include <pango/pangocairo.h>
 #include <cairo-pdf.h>
 #include <vector>
 #include <cmath>
+#include <cstdlib>
+#include <cstring>
 
 namespace gccv {
 
@@ -86,11 +90,40 @@ bool TextPrivate::OnBlink (Text *text)
 	return false;
 }
 
-void TextPrivate::OnCommit (GtkIMContext *context, const gchar *str, Text *text)
+void TextPrivate::OnCommit (G_GNUC_UNUSED GtkIMContext *context, const gchar *str, Text *text)
 {
 	std::string s = str;
-	// FIXME: erase selection
-	text->ReplaceText (s, -1, 0);
+	unsigned start, length;
+	if (text->m_CurPos > text->m_StartSel) {
+		start = text->m_StartSel;
+		length = text->m_CurPos - text->m_StartSel;
+	} else {
+		start = text->m_CurPos;
+		length = text->m_StartSel - text->m_CurPos;
+	}
+	text->ReplaceText (s, start, length);
+}
+
+////////////////////////////////////////////////////////////////////////////////
+//  gccv::TextLine class implementation
+
+class TextLine
+{
+public:
+	TextLine ();
+	~TextLine ();
+
+	double m_Width, m_Height, m_BaseLine;
+	std::list <TextRun *> m_Runs;
+};
+
+TextLine::TextLine ()
+{
+	m_Width = m_Height = m_BaseLine = 0.;
+}
+
+TextLine::~TextLine ()
+{
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -105,8 +138,9 @@ public:
 	void Draw (cairo_t *cr);
 
 	PangoLayout *m_Layout;
-	double m_X, m_Y;
+	double m_X, m_Y, m_Width, m_Height, m_BaseLine;
 	unsigned m_Index, m_Length;
+	bool m_Stacked, m_NewLine;
 };
 
 TextRun::TextRun ()
@@ -114,6 +148,7 @@ TextRun::TextRun ()
 	m_Layout = pango_layout_new (const_cast <PangoContext *> (Ctx.GetContext ()));
 	m_X = m_Y = 0.;
 	m_Index = m_Length = 0;
+	m_Stacked = m_NewLine = false;
 }
 
 TextRun::~TextRun ()
@@ -151,8 +186,7 @@ void TextRun::Draw (cairo_t *cr)
 	PangoLayout *pl = pango_cairo_create_layout (cr);
 	pango_layout_set_font_description (pl, pango_layout_get_font_description (m_Layout));
 	PangoRectangle rect;
-	cairo_set_source_rgba (cr, 0., 0., 0., 1.); // FIXME, use text color if any
-	// FIXME: use text attributes
+	cairo_set_source_rgba (cr, 0., 0., 0., 1.);
 	struct draw_attrs data;
 	while (*text) {
 		pango_layout_iter_get_char_extents (iter, &rect);
@@ -172,7 +206,7 @@ void TextRun::Draw (cairo_t *cr)
 		local = pango_layout_get_iter (pl);
 		pango_layout_iter_get_char_extents (local, &rect);
 		cairo_save (cr);
-		cairo_translate (cr, m_X + curx, m_Y + ascent - (double) pango_layout_iter_get_baseline (local) / PANGO_SCALE);
+		cairo_translate (cr, curx, ascent - (double) pango_layout_iter_get_baseline (local) / PANGO_SCALE);
 		pango_cairo_show_layout (cr, pl);
 		cairo_restore (cr);
 		pango_layout_iter_next_char (iter);
@@ -197,6 +231,7 @@ Text::Text (Canvas *canvas, double x, double y):
 {
 	TextRun *run = new TextRun ();
 	m_Runs.push_front (run);
+	m_FontDesc = pango_font_description_copy (pango_layout_get_font_description (run->m_Layout));
 	m_ImContext = gtk_im_multicontext_new ();
 	g_signal_connect (G_OBJECT (m_ImContext), "commit",
 		G_CALLBACK (TextPrivate::OnCommit), this);
@@ -208,13 +243,18 @@ Text::Text (Group *parent, double x, double y, ItemClient *client):
 	m_BlinkSignal (0),
 	m_CursorVisible (false),
 	m_CurPos (0),
-	m_CurTags (NULL),
+	m_CurTags (new TextTagList ()),
 	m_Padding (0.),
 	m_Anchor (AnchorLine),
-	m_LineOffset (0.), m_Width (0.), m_Height (0.)
+	m_LineOffset (0.),
+	m_Justification (GTK_JUSTIFY_LEFT),
+	m_Width (0.), m_Height (0.)
 {
 	TextRun *run = new TextRun ();
 	m_Runs.push_front (run);
+	m_FontDesc = pango_font_description_copy (pango_layout_get_font_description (run->m_Layout));
+	m_Lines = NULL;
+	m_LinesNumber = 0;
 	m_ImContext = gtk_im_multicontext_new ();
 	g_signal_connect (G_OBJECT (m_ImContext), "commit",
 		G_CALLBACK (TextPrivate::OnCommit), this);
@@ -232,6 +272,9 @@ Text::~Text ()
 	}
 	if (m_CurTags)
 		delete m_CurTags;
+	if (m_Lines)
+		delete [] m_Lines;
+	pango_font_description_free (m_FontDesc);
 }
 
 void Text::SetPosition (double x, double y)
@@ -239,53 +282,33 @@ void Text::SetPosition (double x, double y)
 	double xr, yr, w, h;
 	PangoRectangle r;
 	std::list <TextRun *>::iterator i, end = m_Runs.end ();
-	int x0, y0, x1, y1;
-	if (m_BlinkSignal) {
-		i = m_Runs.begin ();
+	double x0, y0, x1, y1, x_, y_;
+	i = m_Runs.begin ();
+	pango_layout_get_extents ((*i)->m_Layout, NULL, &r);
+	x0 = (*i)->m_X + static_cast <double> (r.x) / PANGO_SCALE;
+	y0 = (*i)->m_Y + static_cast <double> (r.y) / PANGO_SCALE;
+	x1 = x0 + static_cast <double> (r.width) / PANGO_SCALE;
+	y1 = y0 + static_cast <double> (r.height) / PANGO_SCALE;
+	for (i++; i != end; i++) {
 		pango_layout_get_extents ((*i)->m_Layout, NULL, &r);
-		x0 = r.x;
-		y0 = r.y;
-		x1 = x0 + r.width;
-		y1 = y0 + r.height;
-		for (i++; i != end; i++) {
-			pango_layout_get_extents ((*i)->m_Layout, NULL, &r);
-			if (r.x < x0)
-				x0 = r.x;
-			if (r.y < y0)
-				y0 = r.y;
-			r.x += r.width;
-			r.y += r.height;
-			if (r.x > x1)
-				x1 = r.x;
-			if (r.y > y1)
-				y1 = r.y;
-		}
-	} else {
-		i = m_Runs.begin ();
-		pango_layout_get_extents ((*i)->m_Layout, &r, NULL);
-		x0 = r.x;
-		y0 = r.y;
-		x1 = x0 + r.width;
-		y1 = y0 + r.height;
-		for (i++; i != end; i++) {
-			pango_layout_get_extents ((*i)->m_Layout, &r, NULL);
-			if (r.x < x0)
-				x0 = r.x;
-			if (r.y < y0)
-				y0 = r.y;
-			r.x += r.width;
-			r.y += r.height;
-			if (r.x > x1)
-				x1 = r.x;
-			if (r.y > y1)
-				y1 = r.y;
-		}
+		x_ = (*i)->m_X + static_cast <double> (r.x) / PANGO_SCALE;
+		y_ = (*i)->m_Y + static_cast <double> (r.y) / PANGO_SCALE;
+		if (x_ < x0)
+			x0 = x_;
+		if (y_ < y0)
+			y0 = y_;
+		x_ += static_cast <double> (r.width) / PANGO_SCALE;
+		y_ += static_cast <double> (r.height) / PANGO_SCALE;
+		if (x_ > x1)
+			x1 = x_;
+		if (y_ > y1)
+			y1 = y_;
 	}
 	m_x = x;
 	m_y = y;
-	m_Y = (double) y0 / PANGO_SCALE;
-	m_Width = (double) (x0 + x1) / PANGO_SCALE;
-	m_Height = (double) (y1 - y0) / PANGO_SCALE;
+	m_Y = y0;
+	m_Width = x1 - x0;
+	m_Height = y1 - y0;
 	w = m_Width + 2 * m_Padding;
 	h = m_Height + 2 * m_Padding;
 	PangoLayoutIter* iter = pango_layout_get_iter (m_Runs.front ()->m_Layout);
@@ -344,10 +367,9 @@ void Text::Draw (cairo_t *cr, bool is_vector) const
 {
 	Rectangle::Draw (cr, is_vector);
 	// now drawing text
-	// first get the pango iter at first character
-//	PangoLayoutIter* iter = pango_layout_get_iter (m_Layout);
 	// evaluate the starting position
-	double startx, starty, curx;
+	double startx, starty;
+	unsigned start_sel, end_sel;
 	// Horizontal position
 	switch (m_Anchor) {
 	default:
@@ -395,40 +417,51 @@ void Text::Draw (cairo_t *cr, bool is_vector) const
 		starty = m_y - m_Height - m_LineOffset;
 		break;
 	}
+	if (m_CurPos >= m_StartSel) {
+		start_sel = m_StartSel;
+		end_sel = m_CurPos;
+	} else {
+		end_sel = m_StartSel;
+		start_sel = m_CurPos;
+	}
 	// now, draw the glyphs in each run
 	std::list <TextRun *>::const_iterator i, end = m_Runs.end ();
 	for (i = m_Runs.begin (); i != end; i++) {
+		// display the selection rectangle if needed
+		if (m_BlinkSignal) {
+			if (start_sel < (*i)->m_Index + (*i)->m_Length && end_sel > (*i)->m_Index) {
+				PangoRectangle rect;
+				unsigned s = MAX (start_sel, (*i)->m_Index) - (*i)->m_Index,
+						e = MIN (end_sel, (*i)->m_Index + (*i)->m_Length) - (*i)->m_Index;
+				double x, y, w, h;
+				cairo_set_source_rgb (cr, 0.9, 0.9, 0.9);
+				pango_layout_get_cursor_pos ((*i)->m_Layout, s, &rect, NULL);
+				x = (double) rect.x / PANGO_SCALE;
+				h = (double) rect.height / PANGO_SCALE;
+				y = starty + (*i)->m_Y + (double) rect.y / PANGO_SCALE;
+				pango_layout_get_cursor_pos ((*i)->m_Layout, e, &rect, NULL);
+				w = (double) rect.x / PANGO_SCALE - x;
+				x += startx + (*i)->m_X;
+				cairo_rectangle (cr, x, y, w, h);
+				cairo_fill (cr);
+			}
+		}
+		// draw the text
 		cairo_save (cr);
 		cairo_translate (cr, startx + (*i)->m_X, starty + (*i)->m_Y);
 		(*i)->Draw (cr);
 		cairo_restore (cr);
+		// draw the cursor if needed
 		if (m_CursorVisible && m_CurPos > (*i)->m_Index && m_CurPos <= (*i)->m_Index + (*i)->m_Length) {
 			PangoRectangle rect;
 			pango_layout_get_cursor_pos ((*i)->m_Layout, m_CurPos - (*i)->m_Index, &rect, NULL);
 			cairo_set_line_width (cr, 1.);
-			cairo_move_to (cr, floor (startx + (double) rect.x / PANGO_SCALE) + .5, floor (starty + (double) rect.y / PANGO_SCALE) + .5);
+			cairo_move_to (cr, floor (startx + (*i)->m_X + (double) rect.x / PANGO_SCALE) + .5, floor (starty + (*i)->m_Y + (double) rect.y / PANGO_SCALE) + .5);
 			cairo_rel_line_to (cr, 0, rect.height / PANGO_SCALE);
 			cairo_set_source_rgb (cr, 0., 0., 0.);
 			cairo_stroke (cr);
 		}
 	}
-/*	char const *text = pango_layout_get_text (m_Layout);
-	char const *next;
-	PangoLayout *pl = pango_cairo_create_layout (cr);
-	pango_layout_set_font_description (pl, pango_layout_get_font_description (m_Layout));
-	PangoRectangle rect;
-	cairo_set_source_rgba (cr, 0., 0., 0., 1.); // FIXME, use text color if any
-	// FIXME: use text attributes
-	while (*text) {
-		pango_layout_iter_get_char_extents (iter, &rect);
-		curx = rect.x / PANGO_SCALE;
-		cairo_move_to (cr, startx + curx, starty  - m_Ascent + (double) pango_layout_iter_get_baseline (iter) / PANGO_SCALE);
-		next = g_utf8_find_next_char (text, NULL);
-		pango_layout_set_text (pl, text, next - text);
-		text = next;
-		pango_cairo_show_layout (cr, pl);
-		pango_layout_iter_next_char (iter);
-	}*/
 	if (m_CursorVisible && m_CurPos == 0) {
 		PangoRectangle rect;
 		pango_layout_get_cursor_pos (m_Runs.front ()->m_Layout, m_CurPos, &rect, NULL); // FIXME: might be wrong if we allow above-under characters
@@ -439,8 +472,6 @@ void Text::Draw (cairo_t *cr, bool is_vector) const
 		cairo_set_source_rgb (cr, 0., 0., 0.);
 		cairo_stroke (cr);
 	}
-	// free the iterator
-	/*pango_layout_iter_free (iter);*/
 }
 
 void Text::Move (double x, double y)
@@ -457,15 +488,14 @@ void Text::SetText (char const *text)
 {
 	m_Text = text;
 	pango_layout_set_text (m_Runs.front ()->m_Layout, text, -1); // FIXME: parse for line breaks
-	m_Runs.front ()->m_Length = g_utf8_strlen (text, -1);
-	SetPosition (m_x, m_y);
+	m_Runs.front ()->m_Length = strlen (text);
 }
 
 void Text::SetText (std::string const &text)
 {
 	m_Text = text;
 	pango_layout_set_text (m_Runs.front ()->m_Layout, text.c_str (), -1); // FIXME: parse for line breaks
-	m_Runs.front ()->m_Length = g_utf8_strlen (text.c_str (), -1);
+	m_Runs.front ()->m_Length = strlen (text.c_str ());
 	SetPosition (m_x, m_y);
 }
 
@@ -480,10 +510,10 @@ char const *Text::GetText ()
 
 void Text::SetFontDescription (PangoFontDescription *desc)
 {
-	m_FontDesc = desc;
+	m_FontDesc = pango_font_description_copy (desc);
 	std::list <TextRun *>::iterator i, end = m_Runs.end ();
 	for (i = m_Runs.begin (); i != end; i++) {
-		pango_layout_set_font_description ((*i)->m_Layout, desc);
+		pango_layout_set_font_description ((*i)->m_Layout, m_FontDesc);
 	}
 	SetPosition (m_x, m_y);
 }
@@ -508,39 +538,43 @@ void Text::SetEditing (bool editing)
 void Text::GetBounds (Rect *ink, Rect *logical)
 {
 	PangoRectangle i, l;
-	int x0, x1, x2, x3, y0, y1, y2, y3;
+	double x0, x1, x2, x3, y0, y1, y2, y3, x, y;
 	std::list <TextRun *>::iterator it = m_Runs.begin (), end = m_Runs.end ();
 	pango_layout_get_extents ((*it)->m_Layout, &i, &l);
-	x0 = i.x;
-	y0 = i.y;
-	x1 = x0 + i.width;
-	y1 = y0 + i.height;
-	x2 = l.x;
-	y2 = l.y;
-	x3 = x2 + l.width;
-	y3 = y2 + l.height;
+	x0 = (*it)->m_X + static_cast <double> (i.x) / PANGO_SCALE;
+	y0 = (*it)->m_Y + static_cast <double> (i.y) / PANGO_SCALE;
+	x1 = x0 + static_cast <double> (i.width) / PANGO_SCALE;
+	y1 = y0 + static_cast <double> (i.height) / PANGO_SCALE;
+	x2 = (*it)->m_X + static_cast <double> (l.x) / PANGO_SCALE;
+	y2 = (*it)->m_X + static_cast <double> (l.y) / PANGO_SCALE;
+	x3 = x2 + static_cast <double> (l.width) / PANGO_SCALE;
+	y3 = y2 + static_cast <double> (l.height) / PANGO_SCALE;
 	for (it++; it != end; it++) {
 		pango_layout_get_extents ((*it)->m_Layout, &i, &l);
-		if (i.x < x0)
-			x0 = i.x;
-		if (i.y < y0)
-			y0 = i.y;
-		i.x += i.width;
-		i.y += i.height;
-		if (i.x > x1)
-			x1 = i.x;
-		if (i.y > y1)
-			y1 = i.y;
-		if (l.x < x2)
-			x2 = l.x;
-		if (l.y < y2)
-			y2 = l.y;
-		l.x += l.width;
-		l.y += l.height;
-		if (l.x > x3)
-			x3 = l.x;
-		if (l.y > y3)
-			y3 = l.y;
+		x = (*it)->m_X + static_cast <double> (i.x) / PANGO_SCALE;
+		y = (*it)->m_Y + static_cast <double> (i.y) / PANGO_SCALE;
+		if (x < x0)
+			x0 = x;
+		if (y < y0)
+			y0 = y;
+		x += static_cast <double> (i.width) / PANGO_SCALE;
+		y += static_cast <double> (i.height) / PANGO_SCALE;
+		if (x > x1)
+			x1 = x;
+		if (y > y1)
+			y1 = y;
+		x = (*it)->m_X + static_cast <double> (l.x) / PANGO_SCALE;
+		y = (*it)->m_Y + static_cast <double> (l.y) / PANGO_SCALE;
+		if (x < x2)
+			x2 = x;
+		if (y < y2)
+			y2 = y;
+		x += static_cast <double> (l.width) / PANGO_SCALE;
+		y += static_cast <double> (l.height) / PANGO_SCALE;
+		if (x > x3)
+			x3 = x;
+		if (y > y3)
+			y3 = y;
 	}
 	double startx, starty;
 	// Horizontal position
@@ -591,20 +625,20 @@ void Text::GetBounds (Rect *ink, Rect *logical)
 		break;
 	}
 	if (ink) {
-		ink->x0 = startx + (double) x0 / PANGO_SCALE;
-		ink->y0 = starty + (double) y0 / PANGO_SCALE;
-		ink->x1 = ink->x0 + (double) (x1 - x0) / PANGO_SCALE;
-		ink->y1 = ink->y0 + (double) (y1 - y0) / PANGO_SCALE;
+		ink->x0 = startx + x0;
+		ink->y0 = starty + y0;
+		ink->x1 = ink->x0 + x1 - x0;
+		ink->y1 = ink->y0 + y1 - y0;
 	}
 	if (logical) {
-		logical->x0 = startx + (double) x2 / PANGO_SCALE;
-		logical->y0 = starty + (double) y2 / PANGO_SCALE;
-		logical->x1 = logical->x0 + (double) (x3 - x2) / PANGO_SCALE;
-		logical->y1 = logical->y0 + (double) (y3 - y2) / PANGO_SCALE;
+		logical->x0 = startx + x2;
+		logical->y0 = starty + y2;
+		logical->x1 = logical->x0 + x3 - x2;
+		logical->y1 = logical->y0 + y3 - y2;
 	}
 }
 
-void Text::InsertTextTag (TextTag *tag)
+void Text::InsertTextTag (TextTag *tag, bool rebuild_attributes)
 {
 	// FIXME: we need to filter tags to avoid duplicates
 	// now, insert the new tag
@@ -613,20 +647,9 @@ void Text::InsertTextTag (TextTag *tag)
 	else
 		m_Tags.push_back (tag);
 	// now, rebuild pango attributes lists for modified runs.
-	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
-	for (run = m_Runs.begin (); run != end_run; run++) {
-		if ((*run)->m_Index < tag->GetEndIndex () && (*run)->m_Index + (*run)->m_Length > tag->GetStartIndex ()) {
-			PangoAttrList *l = pango_layout_get_attributes ((*run)->m_Layout);
-			if (l == NULL)
-				l = pango_attr_list_new ();
-			unsigned start = (tag->GetStartIndex () > (*run)->m_Index)? tag->GetStartIndex () - (*run)->m_Index: 0;
-			unsigned end = (tag->GetEndIndex () < (*run)->m_Index + (*run)->m_Length)? tag->GetEndIndex () - (*run)->m_Index: (*run)->m_Length;
-			tag->Filter (l, start, end);
-			pango_layout_set_attributes ((*run)->m_Layout, l);	
-		}
-	}
-	// force reposition and redraw
-	SetPosition (m_x, m_y);
+	// FIXME: it would be better to update only the changed portion
+	if (rebuild_attributes)
+		RebuildAttributes ();
 }
 
 void Text::SetCurTagList (TextTagList *l)
@@ -636,8 +659,80 @@ void Text::SetCurTagList (TextTagList *l)
 	m_CurTags = l;
 }
 
-void Text::ApplyCurTagsToSelection ()
+void Text::ApplyTagsToSelection (TextTagList const *l)
 {
+	if (m_CurPos == m_StartSel)
+		return;
+	TextTagList::iterator i, iend = m_Tags.end ();
+	TextTagList::const_iterator j, jend = l->end ();
+	unsigned start, end;
+	if (m_CurPos > m_StartSel) {
+		start = m_StartSel;
+		end = m_CurPos;
+	} else {
+		start = m_CurPos;
+		end = m_StartSel;
+	}
+	TextTagList extra_tags;
+	// store tags to potentially insert into a vector
+	std::vector <TextTag *> new_tags (TextTag::MaxTag);
+	// fist set all vectors to NULL
+	for (int n = 0; n < MaxTag; n++) 
+		new_tags[n] = NULL;
+	// set the tags, unuseful ones will be reset to NULL later
+	for (j = l->begin (); j != jend; j++)
+		new_tags[(*j)->GetTag ()] = *j;
+	for (i = m_Tags.begin (); i != iend; i++) {
+		if ((*i)->GetStartIndex () > end || (*i)->GetEndIndex () < start)
+			continue;
+		for (j = l->begin (); j != jend; j++) {
+			if ((*i)->GetTag() == (*j)->GetTag ()) {
+				if (*(*i) == *(*j)) {
+					// check if we need to change the tag limits
+					if ((*i)->GetStartIndex () > start)
+						(*i)->SetStartIndex (start);
+					if ((*i)->GetEndIndex () < end)
+						(*i)->SetEndIndex (end);
+					// remove *j from new_tags
+					new_tags[(*j)->GetTag ()] = NULL;
+				} else if ((*i)->GetStartIndex () < start) {
+					// we must change the tag limits since they overlap, and possibly split it
+					if ((*i)->GetEndIndex () > end) {
+						// we must split the tag
+						TextTag *tag = (*i)->Duplicate ();
+						tag->SetEndIndex ((*i)->GetEndIndex ());
+						tag->SetStartIndex (end);
+						extra_tags.push_front (tag);
+					}
+					(*i)->SetEndIndex (start);
+				} else // ditto
+					(*i)->SetStartIndex (end);
+			}
+		}
+	}
+	// Add new tags
+	for (int n = 0; n < TextTag::MaxTag; n++) 
+		if (new_tags[n]) {
+			TextTag *tag = new_tags[n]->Duplicate ();
+			tag->SetStartIndex (start);
+			tag->SetEndIndex (end);
+			if (tag->GetPriority () == TagPriorityFirst)
+				m_Tags.push_front (tag);
+			else
+				m_Tags.push_back (tag);
+		}
+	
+	// Add extra tags
+	iend = extra_tags.end ();
+	for (i = extra_tags.begin (); i != iend; i++)
+		if ((*i)->GetPriority () == TagPriorityFirst)
+			m_Tags.push_front (*i);
+		else
+			m_Tags.push_back (*i);
+	extra_tags.clear (); // avoid destroying the tags
+	// Force redraw
+	RebuildAttributes ();
+	SetPosition (m_x, m_y);
 }
 
 void Text::ReplaceText (std::string &str, int pos, unsigned length)
@@ -653,10 +748,9 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 	TextTagList new_tags, extra_tags;
 	if (length > 0) {
 		m_Text.erase (pos, length);
-		std::vector <TextTag *> borders (MaxTag);
-		for (int n = 0; n < MaxTag; n++) 
+		std::vector <TextTag *> borders (TextTag::MaxTag);
+		for (int n = 0; n < TextTag::MaxTag; n++) 
 			borders[n] = NULL;
-		//TODO: manage attributes
 		//TODO: update runs
 		for (i = m_Tags.begin (); i != iend; i++) {
 			unsigned end = (*i)->GetEndIndex (), start = (*i)->GetStartIndex ();
@@ -732,7 +826,7 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 		pango_layout_set_text (m_Runs.front ()->m_Layout, m_Text.c_str (), -1); // FIXME: parse for line breaks and update runs
 		m_Runs.front ()->m_Length = m_Text.length ();
 		RebuildAttributes ();
-		m_CurPos = pos;
+		m_CurPos = m_StartSel = pos;
 		SetPosition (m_x, m_y);
 		//TODO: update attribute list
 		return;
@@ -770,7 +864,6 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 		}
 	}
 	// now add all new tags to the list
-	// first those from new_tags:
 	iend = new_tags.end ();
 	for (i = new_tags.begin (); i != iend; i++)
 		if ((*i)->GetPriority () == TagPriorityFirst)
@@ -793,14 +886,20 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 	pango_layout_set_text (m_Runs.front ()->m_Layout, m_Text.c_str (), -1); // FIXME: parse for line breaks and update runs
 	m_Runs.front ()->m_Length = m_Text.length ();
 	RebuildAttributes ();
-	m_CurPos = pos + str.length ();
+	m_CurPos = m_StartSel = pos + str.length ();
 	SetPosition (m_x, m_y);
 }
 
+static std::string empty_st = "";
+
 bool Text::OnKeyPressed (GdkEventKey *event)
 {
-	if (gtk_im_context_filter_keypress (m_ImContext, event))
+	TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+	if (gtk_im_context_filter_keypress (m_ImContext, event)) {
+		if (client)
+			client->TextChanged (m_CurPos);
 		return true;
+	}
 
 	switch (event->keyval) {
 	case GDK_Control_L:
@@ -813,35 +912,66 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 
 	case GDK_Tab:
 		TextPrivate::OnCommit (m_ImContext, "\t", this);
+		if (client)
+			client->TextChanged (m_CurPos);
 		break;
 
 	/* MOVEMENT */
 	case GDK_Right:
-		/* TODO: write this code */
+		if (m_CurPos == m_Text.length ())
+			break;
 		if (event->state & GDK_CONTROL_MASK) {
-			/* move to start of word */
-			/* TODO: write this code */
+			/* move to end of word */
+			char const* s = m_Text.c_str ();
+			char *p = g_utf8_next_char (s + m_CurPos);
+			while (*p && (!g_unichar_isgraph (g_utf8_get_char(p)) || g_unichar_ispunct (g_utf8_get_char(p))))
+				p = g_utf8_next_char (p);
+			while (g_unichar_isgraph (g_utf8_get_char(p)) && !g_unichar_ispunct (g_utf8_get_char(p)))
+				p = g_utf8_next_char (p);
+			m_CurPos = p - s;
+			if (!(event->state & GDK_SHIFT_MASK))
+				m_StartSel = m_CurPos;
+			Invalidate ();
 		} else {
 			char const* s = m_Text.c_str ();
 			char *p = g_utf8_next_char (s + m_CurPos);
 			if (!p)
 				break;
 			m_CurPos = p - s;
+			if (!(event->state & GDK_SHIFT_MASK))
+				m_StartSel = m_CurPos;
 			Invalidate ();
 		}
-			break;
+		if (client)
+			client->SelectionChanged (m_StartSel, m_CurPos);
+		break;
 	case GDK_Left:
 		if (m_CurPos == 0)
 			break;
 		if (event->state & GDK_CONTROL_MASK) {
 			/* move to start of word */
-			/* TODO: write this code */
+			char const* s = m_Text.c_str ();
+			char *p = g_utf8_prev_char (s + m_CurPos);
+			while (p != s && (!g_unichar_isgraph (g_utf8_get_char(p)) || g_unichar_ispunct (g_utf8_get_char (p))))
+				p = g_utf8_prev_char (p);
+			while (p != s && g_unichar_isgraph (g_utf8_get_char(p)) && !g_unichar_ispunct (g_utf8_get_char(p)))
+				p = g_utf8_prev_char (p);
+			if (!g_unichar_isgraph (g_utf8_get_char(p))) // don't go to the previous word end
+				p = g_utf8_next_char (p);
+			m_CurPos = p - s;
+			if (!(event->state & GDK_SHIFT_MASK))
+				m_StartSel = m_CurPos;
+			Invalidate ();
 		} else {
 			char const* s = m_Text.c_str ();
 			char *p = g_utf8_prev_char (s + m_CurPos);
 			m_CurPos = p - s;
+			if (!(event->state & GDK_SHIFT_MASK))
+				m_StartSel = m_CurPos;
 			Invalidate ();
 		}
+		if (client)
+			client->SelectionChanged (m_StartSel, m_CurPos);
 		break;
 	case GDK_f:
 		/* TODO: write this code */
@@ -879,20 +1009,40 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 	/* DELETING TEXT */
 	case GDK_Delete:
 	case GDK_KP_Delete: {
-		/* TODO: write this code */
+		if (m_CurPos != m_StartSel) {
+			ReplaceText (empty_st, MIN (m_CurPos, m_StartSel), abs (m_CurPos - m_StartSel));
+			if (client)
+				client->TextChanged (m_CurPos);
+			break;
+		}
+		if (m_CurPos == m_Text.length ())
+			break;
+		char const* s = m_Text.c_str ();
+		char *p = g_utf8_next_char (s + m_CurPos);
+		int new_pos = p - s;
+		ReplaceText (empty_st, m_CurPos, new_pos - m_CurPos);
+		if (client)
+			client->TextChanged (m_CurPos);
+		break;
 	}
 	case GDK_d:
 		/* TODO: write this code */
 		break;
 	case GDK_BackSpace: {
-		/* TODO: write this code */
+		if (m_CurPos != m_StartSel) {
+			ReplaceText (empty_st, MIN (m_CurPos, m_StartSel), abs (m_CurPos - m_StartSel));
+			if (client)
+				client->TextChanged (m_CurPos);
+			break;
+		}
 		if (m_CurPos == 0)
 			break;
 		char const* s = m_Text.c_str ();
 		char *p = g_utf8_prev_char (s + m_CurPos);
 		int new_pos = p - s;
-		std::string st = "";
-		ReplaceText (st, new_pos, m_CurPos - new_pos);
+		ReplaceText (empty_st, new_pos, m_CurPos - new_pos);
+		if (client)
+			client->TextChanged (m_CurPos);
 		break;
 	}
 	case GDK_k:
@@ -921,9 +1071,53 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 
 void Text::RebuildAttributes ()
 {
+	// This might be optimized: not everything should be scanned each time a change occur
+	// first update runs list
+	// we need to order tags
+	m_Tags.sort (gccv::TextTag::Order);
 	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
+	for (run = m_Runs.begin ();run != end_run; run++) // delete all runs
+		delete (*run);
+	m_Runs.clear ();
+	// Recreate first run
+	TextRun *new_run = new TextRun (), *last_run;
+	pango_layout_set_font_description (new_run->m_Layout, m_FontDesc);
+	m_Runs.push_front (new_run);
+	last_run = new_run;
 	TextTagList::iterator tag, end_tag = m_Tags.end ();
+	bool stacked = false;
+	unsigned lines = 1;
+	std::string str;
+	for (tag = m_Tags.begin (); tag != end_tag; tag++) {
+		if (stacked || (*tag)->GetStacked ()) {
+			// we need a new run
+			new_run = new TextRun ();
+			pango_layout_set_font_description (new_run->m_Layout, m_FontDesc);
+			new_run ->m_Index = (*tag)->GetStartIndex ();
+			last_run->m_Length = new_run->m_Index - last_run->m_Index;
+			stacked = new_run->m_Stacked = (*tag)->GetStacked ();
+			m_Runs.push_back (new_run);
+			last_run = new_run;
+		} else if ((*tag)->GetNewLine ()) {
+			lines++;
+			stacked = false;
+			// we need a new run
+			new_run = new TextRun ();
+			pango_layout_set_font_description (new_run->m_Layout, m_FontDesc);
+			new_run ->m_Index = (*tag)->GetStartIndex ();
+			last_run->m_Length = new_run->m_Index - last_run->m_Index;
+			new_run->m_NewLine = true;
+			m_Runs.push_back (new_run);
+			last_run = new_run;
+		}
+	}
+	last_run->m_Length = m_Text.length () - last_run->m_Index;
+	// now update attributes for each run
+	end_run = m_Runs.end ();
+	PangoLayoutIter *iter;
 	for (run = m_Runs.begin (); run != end_run; run++) {
+		str = m_Text.substr ((*run)->m_Index, (*run)->m_Length);
+		pango_layout_set_text ((*run)->m_Layout, str.c_str (), -1);
 		PangoAttrList *l = pango_attr_list_new ();
 		for (tag = m_Tags.begin (); tag != end_tag; tag++) {
 			if ((*tag)->GetEndIndex () <= (*run)->m_Index || (*tag)->GetStartIndex () >= (*run)->m_Index + (*run)->m_Length)
@@ -934,9 +1128,127 @@ void Text::RebuildAttributes ()
 		}
 		pango_layout_set_attributes ((*run)->m_Layout, l);
 		pango_attr_list_unref (l);
+		PangoRectangle rect;
+		pango_layout_get_extents ((*run)->m_Layout, NULL, &rect);
+		(*run)->m_Width = (double) rect.width / PANGO_SCALE;
+		(*run)->m_Height = (double) rect.height / PANGO_SCALE;
+		iter = pango_layout_get_iter ((*run)->m_Layout);
+		(*run)->m_BaseLine = (double) pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		pango_layout_iter_free (iter);
+	}
+	if (m_Lines)
+		delete [] m_Lines;
+	m_Lines = new TextLine[lines];
+	m_LinesNumber = lines;
+	// we now need to rebuild runs positions
+	// FIXME: support several lines
+	double curx = 0., curw = 0.;
+	unsigned cur_line = 0;
+	for (run = m_Runs.begin (); run != end_run; run++) {
+		m_Lines[cur_line].m_Runs.push_back (*run);
+		if ((*run)->m_Stacked) {
+			if ((*run)->m_Width > curw)
+				curw = (*run)->m_Width;
+			(*run)->m_X = curx;
+		} else if ((*run)->m_NewLine) {
+			m_Lines[cur_line].m_Width = curx + curw;
+			cur_line++;
+		} else {
+			(*run)->m_X = curx + curw;
+			curx += (*run)->m_Width;
+			curw = 0.;
+		}
+		if (m_Lines[cur_line].m_BaseLine < (*run)->m_BaseLine)
+			m_Lines[cur_line].m_BaseLine = (*run)->m_BaseLine;
+	}
+	m_Lines[cur_line].m_Width = curx + curw;
+	for (cur_line = 0; cur_line < lines; cur_line++) {
+		end_run = m_Lines[cur_line].m_Runs.end ();
+		for (run = m_Lines[cur_line].m_Runs.begin (); run != end_run; run++)
+			(*run)->m_Y = m_Lines[cur_line].m_BaseLine - (*run)->m_BaseLine;
 	}
 	// force reposition and redraw
 	SetPosition (m_x, m_y);
 }
 
+void Text::OnButtonPressed (double x, double y)
+{
+	double x0 = m_x0, y0 = m_y0, x1 = 0., y1 = 0.;
+	GetParent ()->AdjustBounds (x0, y0, x1, y1);
+	x -= x0;
+	y -= y0;
+	int index, trailing;
+	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
+	for (run = m_Runs.begin (); run != end_run; run++)
+		if (pango_layout_xy_to_index ((*run)->m_Layout, x * PANGO_SCALE, y * PANGO_SCALE, &index, &trailing)) {
+			m_CurPos = m_StartSel = index + trailing + (*run)->m_Index;
+			TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+			if (client)
+				client->SelectionChanged (m_StartSel, m_CurPos);
+			return;
+		}
+}
+
+void Text::OnDrag (double x, double y)
+{
+	int index, trailing;
+	x -= m_x0;
+	y -= m_y0;
+	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
+	for (run = m_Runs.begin (); run != end_run; run++)
+		if (pango_layout_xy_to_index ((*run)->m_Layout, x * PANGO_SCALE, y * PANGO_SCALE, &index, &trailing)) {
+			m_CurPos = index + trailing + (*run)->m_Index;
+			Invalidate ();
+			TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+			if (client)
+				client->SelectionChanged (m_StartSel, m_CurPos);
+			return;
+		}
+}
+
+void Text::GetSelectionBounds (unsigned &start, unsigned &end)
+{
+	start = m_StartSel;
+	end = m_CurPos;
+}
+
+void Text::SetSelectionBounds (unsigned start, unsigned end)
+{
+	GetText (); // force update of m_Text
+	m_StartSel = (start > m_Text.length ())? m_Text.length (): start;
+	m_CurPos = (end > m_Text.length ())? m_Text.length (): end;
+	TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+	if (client)
+		client->SelectionChanged (m_StartSel, m_CurPos);
+	Invalidate ();
+}
+
+unsigned Text::GetIndexAt (double x, double y)
+{
+	int index, trailing;
+	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
+	for (run = m_Runs.begin (); run != end_run; run++)
+		if (pango_layout_xy_to_index ((*run)->m_Layout, x * PANGO_SCALE, y * PANGO_SCALE, &index, &trailing))
+			return index + trailing + (*run)->m_Index;
+	return G_MAXUINT;
+}
+
+bool Text::GetPositionAtIndex (unsigned index, Rect &rect)
+{
+	if (index > m_Text.length ())
+		return false;
+	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
+	for (run = m_Runs.begin (); run != end_run; run++)
+		if (index <= (*run)->m_Index + (*run)->m_Length) {
+			PangoRectangle r;
+			pango_layout_index_to_pos ((*run)->m_Layout, index - (*run)->m_Index, &r);
+			rect.x0 = (double) r.x / PANGO_SCALE;
+			rect.y0 = (double) r.y / PANGO_SCALE;
+			rect.x1 = (double) (r.x + r.width) / PANGO_SCALE;
+			rect.y1 = (double) (r.y + r.height) / PANGO_SCALE;
+			break;
+		}
+	return true;
+}
+
 }
