@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gccv/text.h 
  *
- * Copyright (C) 2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2008-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -49,6 +49,7 @@ private:	\
 
 // These are private classes
 class TextPrivate;
+class TextLine;
 class TextRun;
 class TextTag;
 class TextTagList;
@@ -70,12 +71,12 @@ public:
 	void GetBounds (Rect *ink, Rect *logical);
 	char const *GetText ();
 
-	void InsertTextTag (TextTag *tag);
+	void InsertTextTag (TextTag *tag, bool rebuild_attributes = true);
 	std::list <TextTag *> const *GetTags () {return &m_Tags;}
 
 	void SetCurTagList (TextTagList *l);
 
-	void ApplyCurTagsToSelection ();
+	void ApplyTagsToSelection (TextTagList const *l);
 
 /*!
 @param str the text to insert
@@ -88,6 +89,10 @@ be moved to the end of the inserted text.
 */
 	void ReplaceText (std::string &str, int pos, unsigned length);
 	unsigned GetCursorPosition () {return m_CurPos;}
+	void GetSelectionBounds (unsigned &start, unsigned &end);
+	void SetSelectionBounds (unsigned start, unsigned end);
+	unsigned GetIndexAt (double x, double y);
+	bool GetPositionAtIndex (unsigned index, Rect &rect);
 
 	// virtual methods
 	void Draw (cairo_t *cr, bool is_vector) const;
@@ -95,31 +100,33 @@ be moved to the end of the inserted text.
 
 	// events related methods
 	bool OnKeyPressed (GdkEventKey *event);
+	void OnButtonPressed (double x, double y);
+	void OnDrag (double x, double y);
 
 	// static methods
 	static PangoContext *GetContext ();
 
 	unsigned GetDefaultFontSize () { return (m_FontDesc)? (double) pango_font_description_get_size (m_FontDesc) / PANGO_SCALE: 0; }
-
-private:
 	void RebuildAttributes ();
 
 private:
 	double m_x, m_y;
 	unsigned long m_BlinkSignal;
 	bool m_CursorVisible;
-	unsigned m_CurPos;
+	unsigned m_CurPos, m_StartSel;
 	std::list <TextRun *> m_Runs;
 	std::list <TextTag *> m_Tags;
 	std::string m_Text;
 	GtkIMContext *m_ImContext;
 	PangoFontDescription *m_FontDesc;
 	TextTagList *m_CurTags;
-
+	TextLine *m_Lines;
+	unsigned m_LinesNumber;
 
 GCCV_TEXT_PROP (double, Padding)
 GCCV_TEXT_PROP (Anchor, Anchor)
 GCCV_TEXT_PROP (double, LineOffset)
+GCCV_TEXT_PROP (GtkJustification, Justification)
 GCU_RO_PROP (double, Width)
 GCU_RO_PROP (double, Height)
 GCU_RO_PROP (double, Ascent)
