@@ -4,7 +4,7 @@
  * GChemPaint library
  * application.h 
  *
- * Copyright (C) 2004-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,18 +26,18 @@
 #define GCHEMPAINT_APPLICATION_H
 
 #include <gcu/application.h>
-#include <gcu/dialog.h>
 #include <gcu/macros.h>
-#include <gcu/object.h>
-#ifdef HAVE_GO_CONF_SYNC
-#include <goffice/app/go-conf.h>
-#else
-#include <gconf/gconf-client.h>
-#endif
+#include <libxml/parser.h>
 #include <set>
 #include <string>
 #include <map>
 #include <list>
+#include <stdexcept>
+
+namespace gcu {
+	class Dialog;
+	class Object;
+}
 
 /*!\file*/
 /*!\namespace gcp
@@ -71,6 +71,12 @@ class Document;
 struct option_data;
 typedef void (*BuildMenuCb) (GtkUIManager *UIManager);
 
+typedef enum {
+	CursorUnallowed,
+	CursorPencil,
+	CursorMax
+} CursorId;
+
 /*!\class Application gcp/application.h
 \brief GChemPaint application base class.
 
@@ -352,7 +358,6 @@ Closes all open documents and ends the application.
 @return a list of supported mime types.
 */
 	std::list<std::string> &GetSupportedMimeTypes () {return m_SupportedMimeTypes;}
-#ifdef HAVE_GO_CONF_SYNC
 /*!
 @param node the GONode which changed.
 @param name the name of the key.
@@ -361,17 +366,6 @@ Called by the framework when the configuration entry has changed to update a
 running application preferences if the system allows such callbacks.
 */
 	void OnConfigChanged (GOConfNode *node, gchar const *name);
-#else
-/*!
-@param client a GConfClient.
-@param cnxn_id the id returned by gconf_client_notify_add.
-@param entry the entry which changed.
-
-Called by the framework when the configuration entry has changed to update a
-running application preferences.
-*/
-	void OnConfigChanged (GConfClient *client, guint cnxn_id, GConfEntry *entry);
-#endif
 /*!
 @param mime_type a mime type.
 @return the list of file name extensions corresponding to the mime type.
@@ -432,6 +426,8 @@ it is pure virtual.
 */
 	virtual void OnFileNew (char const *Theme = NULL) = 0;
 
+	GdkCursor *GetCursor (CursorId id)  {return m_Cursors[id];}
+
 protected:
 /*!
 Initialize the tools box so that the selection tool is active. This method is
@@ -442,7 +438,7 @@ called only once aafter startup by the framework.
 Builds the tools box. This method is
 called only once after startup by the framework.
 */
-	void BuildTools ();
+	void BuildTools () throw (std::runtime_error);
 /*!
 @param visible whether the tools box should be visible or not
 
@@ -488,15 +484,12 @@ private:
 	int visible_windows;
 	std::list<std::string> m_SupportedMimeTypes;
 	std::list<std::string> m_WriteableMimeTypes;
-#ifdef HAVE_GO_CONF_SYNC
 	GOConfNode *m_ConfNode;
-#else
-	GConfClient *m_ConfClient;
-#endif
 	guint m_NotificationId;
 	gcu::Object *m_Dummy;
 	std::list<BuildMenuCb> m_MenuCbs;
 	std::list<option_data> m_Options;
+	GdkCursor *m_Cursors[CursorMax];
 
 GCU_RO_POINTER_PROP	(GtkStyle, Style)
 };
