@@ -130,13 +130,13 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 		if (ax > 0) {
 			horizontal = anga >= angb;
 			if (horizontal) {
-				ax = (ax + 2.) / Theme->GetZoomFactor ();
+				ax = (ax + 1.) / Theme->GetZoomFactor ();
 				if (dx > 0)
 					ax = - ax;
 				*x2 += ax;
 				*y2 += ax * dy / dx;
 			} else {
-				ay = (ay + 2.) / Theme->GetZoomFactor ();
+				ay = (ay + 1.) / Theme->GetZoomFactor ();
 				if (dy > 0)
 					ay = - ay;
 				*y2 += ay;
@@ -259,8 +259,8 @@ general:
 								}
 								bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
 							}
-						m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
-						m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
+						m_coords[4] = *x1 - BondDist * cosa * tanb - dy * sign;
+						m_coords[5] = *y1 + dx * sign + BondDist * sina * tanb;
 						tanb = (m_End->GetZ () == 6)? fabs (tan ((a1 - a0) / 2)): 0.;
 						m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
 						m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
@@ -361,7 +361,7 @@ done:
 	return true;
 }
 
-Object* Bond::GetAtomAt(double x, double y, double z)
+Object* Bond::GetAtomAt(double x, double y, G_GNUC_UNUSED double z)
 {
 	double x1, y1;
 	m_Begin->GetCoords (&x1, &y1);
@@ -373,7 +373,7 @@ Object* Bond::GetAtomAt(double x, double y, double z)
 	return NULL;
 }
 
-bool Bond::SaveNode (xmlDocPtr xml, xmlNodePtr node) const
+bool Bond::SaveNode (G_GNUC_UNUSED xmlDocPtr xml, xmlNodePtr node) const
 {
 	switch(m_type) {
 	case UpBondType:
@@ -498,7 +498,7 @@ void Bond::RemoveAllCycles ()
 	}
 }
 
-void Bond::Move (double x, double y, double z)
+void Bond::Move (double x, double y, G_GNUC_UNUSED double z)
 {
 	if (m_Item) {
 		Document *doc = static_cast <Document*> (GetDocument ());
@@ -508,7 +508,7 @@ void Bond::Move (double x, double y, double z)
 	m_CoordsCalc = false;
 }
 
-void Bond::Transform2D (Matrix2D& m, double x, double y)
+void Bond::Transform2D (G_GNUC_UNUSED Matrix2D& m, G_GNUC_UNUSED double x, G_GNUC_UNUSED double y)
 {
 	m_CoordsCalc = false;
 }
@@ -1193,6 +1193,8 @@ bool Bond::SetProperty (unsigned property, char const *value)
 			BondTypesValues["wedge"] = s;
 			s.type = DownBondType;
 			BondTypesValues["hash"] = s;
+			s.type = UndeterminedBondType;
+			BondTypesValues["unknown"] = s;
 			s.invert = true;
 			s.type = UpBondType;
 			BondTypesValues["wedge-invert"] = s;
@@ -1213,4 +1215,28 @@ bool Bond::SetProperty (unsigned property, char const *value)
 	return  true;
 }
 
+string Bond::GetProperty (unsigned property) const
+{
+	switch (property) {
+	case GCU_PROP_BOND_TYPE: {
+		switch (m_type) {
+		case NormalBondType:
+		default:
+			return "normal";
+		case UpBondType:
+			return "wedge";
+		case DownBondType:
+			return "hash";
+		case ForeBondType:
+			return "bold";
+		case UndeterminedBondType:
+			return "unknown";
+
+		}
+	}
+	default:
+		return gcu::Bond::GetProperty (property);
+	}
+}
+
 }	//	namespace gcp
