@@ -42,9 +42,6 @@
 #include "window.h"
 #include <gcu/loader.h>
 #include <gcu/objprops.h>
-#include <goffice/math/go-rangefunc.h>
-#include <gtk/gtk.h>
-#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
@@ -513,10 +510,10 @@ void Document::AddFragment (Fragment* pFragment)
 		pFragment->SetId (id);
 		Id = id;
 	}
+	AddObject(pFragment);
 	m_pView->AddObject (pFragment);
 	if (m_bIsLoading)
 		return;
-//	AddObject(pFragment);
 	if (!pFragment->GetMolecule ()) {
 		Molecule* mol = new Molecule ();
 		i = 1;
