@@ -4,7 +4,7 @@
  * GChemPaint library
  * fragment.cc 
  *
- * Copyright (C) 2002-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,20 +23,23 @@
  */
 
 #include "config.h"
+#include "application.h"
+#include "bond.h"
+#include "document.h"
 #include "fragment.h"
 #include "fragment-residue.h"
-#include "widgetdata.h"
-#include "document.h"
-#include "application.h"
-#include "view.h"
+#include "settings.h"
 #include "text.h"
 #include "theme.h"
 #include "tool.h"
-#include "settings.h"
+#include "view.h"
+#include "widgetdata.h"
 #include "window.h"
-#include <canvas/gcp-canvas-group.h>
-#include <canvas/gcp-canvas-rect-ellipse.h>
-#include <canvas/gcp-canvas-bpath.h>
+#include <gccv/canvas.h>
+#include <gccv/group.h>
+#include <gccv/structs.h>
+#include <gccv/text.h>
+#include <gccv/text-tag.h>
 #include <gcu/element.h>
 #include <gcu/formula.h>
 #include <gcu/objprops.h>
@@ -51,19 +54,33 @@ using namespace std;
 
 namespace gcp {
 
-static void on_fragment_changed (Fragment *fragment)
+gccv::Tag ChargeTag = gccv::Invalid, StoichiometryTag = gccv::Invalid;
+
+ChargeTextTag::ChargeTextTag (double size):
+	gccv::PositionTextTag (gccv::Superscript, size, true, (ChargeTag)? ChargeTag:(ChargeTag = gccv::TextTag::RegisterTagType ()))
 {
-	fragment->OnChanged (true);
 }
 
-static void on_fragment_sel_changed (Fragment *fragment, struct GnomeCanvasPangoSelBounds *bounds)
+ChargeTextTag::~ChargeTextTag ()
 {
-	fragment->OnSelChanged (bounds);
 }
 
+StoichiometryTextTag::StoichiometryTextTag (double size):
+	gccv::PositionTextTag (gccv::Subscript, size, true, (StoichiometryTag)? StoichiometryTag:(StoichiometryTag = gccv::TextTag::RegisterTagType ()))
+{
+}
+
+StoichiometryTextTag::~StoichiometryTextTag ()
+{
+}
+
+// FIXME: search for unuseful things, such as m_CHeight? is it really unused?
 Fragment::Fragment ():
 	TextObject (FragmentType),
-	m_Inversable (false)
+	m_Inversable (false),
+	m_TextItem (NULL),
+	m_Valid (Invalid),
+	m_Mode (AutoMode)
 {
 	m_Atom = new FragmentAtom (this, 0);
 	m_BeginAtom = m_EndAtom = 0;
@@ -75,7 +92,10 @@ Fragment::Fragment ():
 
 Fragment::Fragment (double x, double y):
 	TextObject (x, y, FragmentType),
-	m_Inversable (false)
+	m_Inversable (false),
+	m_TextItem (NULL),
+	m_Valid (Invalid),
+	m_Mode (AutoMode)
 {
 	m_Atom = new FragmentAtom (this, 0);
 	m_Atom->SetCoords (x, y);
@@ -93,38 +113,37 @@ Fragment::~Fragment ()
 
 bool Fragment::OnChanged (bool save)
 {
-// FIXME: this code is full of bugs: rewrite it
 	if (m_bLoading)
 		return false;
 	Document* pDoc = (Document*) GetDocument ();
 	if (!pDoc)
 		return false;
+	m_buf = m_TextItem->GetText ();
 	View* pView = pDoc->GetView ();
-	GtkWidget* pWidget = pView->GetWidget ();
+/*	GtkWidget* pWidget = pView->GetWidget ();
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (pWidget), "data");
 	GnomeCanvasGroup *group = pData->Items[this];
 	if (!group) {
 		pData->Items.erase (this);
 		m_bLoading = false;
 		return false;
-	}
-	GnomeCanvasPango *PangoItem = GNOME_CANVAS_PANGO (g_object_get_data (G_OBJECT (group), "fragment"));
-	unsigned CurPos = gnome_canvas_pango_get_cur_index (PangoItem);
-	AnalContent (m_StartSel, CurPos);
+	}*/
+	unsigned CurPos = m_TextItem->GetCursorPosition ();
+//	AnalContent (m_StartSel, CurPos);//why? Probably not anymore needed
 	m_bLoading = true;
-	m_buf = pango_layout_get_text (m_Layout);
-	if (m_buf.length ()) {
+	// Get the ascent, is it still needed now that we can align texts according to the baseline
+/*	if (m_buf.length ()) {
 		PangoLayoutIter *iter = pango_layout_get_iter (m_Layout);
 		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 		pango_layout_iter_free (iter);
-	}
+	}*/
+	/*main atom management*/
 	if (m_buf.length () < m_EndAtom) { // needed if the symbol of part of it has been destroyed
 		m_Atom->SetZ (0);
 		m_EndAtom = m_buf.length ();
 		if (m_BeginAtom > m_EndAtom)
 			m_BeginAtom = m_EndAtom;
 	}
-	/*main atom management*/
 	FragmentResidue *residue = dynamic_cast <FragmentResidue*> (m_Atom);
 	Residue *r = NULL;
 	char sy[Residue::MaxSymbolLength + 1];
@@ -253,14 +272,15 @@ bool Fragment::OnChanged (bool save)
 				m_EndAtom = CurPos;
 		}
 	}
-	PangoRectangle rect;
-	pango_layout_index_to_pos (m_Layout, m_BeginAtom, &rect);
-	m_lbearing = rect.x / PANGO_SCALE;
-	pango_layout_index_to_pos (m_Layout, m_EndAtom, &rect);
-	m_lbearing += rect.x / PANGO_SCALE;
+	gccv::Rect rect;
+	m_TextItem->GetPositionAtIndex (m_BeginAtom, rect);
+	m_lbearing = rect.x0;
+	m_TextItem->GetPositionAtIndex (m_EndAtom, rect);
+	m_lbearing += rect.x0;
 	m_lbearing /=  2;
 	pView->Update (this);
 	m_bLoading = false;
+	m_Valid = Invalid;
 	Window* pWin = pDoc->GetWindow ();
 	if (m_Atom->GetZ () || ((m_buf.length () == 0) && (m_Atom->GetBondsNumber () == 0))) {
 		if (!pDoc->GetReadOnly ()) {
@@ -275,12 +295,12 @@ bool Fragment::OnChanged (bool save)
 		pWin->ActivateActionWidget ("/MainMenu/FileMenu/Print", false);
 		pWin->ActivateActionWidget ("/MainToolbar/Save", false);
 	}
-	pango_layout_get_extents (m_Layout, NULL, &rect);
+/*	pango_layout_get_extents (m_Layout, NULL, &rect);
 	m_length = rect.width / PANGO_SCALE;
 	m_height = rect.height / PANGO_SCALE;
 	pView->Update (this);
+	m_StartSel = m_EndSel = CurPos;*/
 	EmitSignal (OnChangedSignal);
-	m_StartSel = m_EndSel = CurPos;
 	if (m_buf.length () == 0) {
 		m_BeginAtom = m_EndAtom = 0;
 	}
@@ -295,7 +315,55 @@ bool Fragment::OnChanged (bool save)
 	return true;
 }
 
-void Fragment::Add (GtkWidget* w) const
+void Fragment::AddItem ()
+{
+	if (m_Item)
+		return;
+	Document *doc = static_cast <Document*> (GetDocument ());
+	View *view = doc->GetView ();
+	Theme *theme = doc->GetTheme ();
+	PangoFontDescription *desc = view->GetPangoFontDesc ();
+	if (m_ascent <= 0) {
+		PangoContext* pc = gccv::Text::GetContext ();
+		PangoLayout *layout = pango_layout_new (pc);
+		pango_layout_set_font_description (layout, desc);
+		PangoAttrList *l = pango_attr_list_new ();
+		pango_layout_set_attributes (layout, l);
+		pango_layout_set_font_description (layout, desc);
+		pango_layout_set_text (layout, "l", -1);
+		PangoLayoutIter* iter = pango_layout_get_iter (layout);
+		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		pango_layout_iter_free (iter);
+	}
+	double x = m_x * theme->GetZoomFactor ();
+	double y = m_y * theme->GetZoomFactor ();
+	gccv::Group *group = new gccv::Group (view->GetCanvas ()->GetRoot (), x, y, this);
+	m_TextItem = new gccv::Text (group, 0., 0., this);
+	m_TextItem->SetFillColor ((view->GetData ()->IsSelected (this))? SelectColor: 0);
+	m_TextItem->SetPadding (theme->GetPadding ());
+	m_TextItem->SetLineColor (0);
+	m_TextItem->SetLineOffset (view->GetCHeight ());
+	m_TextItem->SetAnchor (gccv::AnchorLineWest);
+	m_TextItem->SetFontDescription (desc);
+	m_TextItem->SetText (m_buf);
+	while (!m_TagList.empty ()) {
+		m_TextItem->InsertTextTag (m_TagList.front (), false);
+		m_TagList.pop_front ();
+	}
+	m_TextItem->RebuildAttributes ();
+	m_Item = group;
+}
+
+void Fragment::UpdateItem ()
+{
+	if (!m_TextItem)
+		return;
+	Theme *theme = static_cast <Document*> (GetDocument ())->GetTheme ();
+	static_cast <gccv::Group *> (m_Item)->SetPosition (m_x * theme->GetZoomFactor (), m_y * theme->GetZoomFactor ());
+	m_TextItem->SetPosition (-m_lbearing, 0);
+}
+
+/*void Fragment::Add (GtkWidget* w) const
 {
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	if (pData->Items[this] != NULL)
@@ -303,7 +371,7 @@ void Fragment::Add (GtkWidget* w) const
 	View* pView = pData->m_View;
 	Theme *pTheme = pView->GetDoc ()->GetTheme ();
 	if (m_ascent <= 0) {
-		PangoContext* pc = pView->GetPangoContext ();
+		PangoContext* pc = gccv::Text::GetContext ();
 		const_cast <Fragment *> (this)->m_Layout = pango_layout_new (pc);
 		PangoAttrList *l = pango_attr_list_new ();
 		pango_layout_set_attributes (m_Layout, l);
@@ -362,9 +430,9 @@ void Fragment::Add (GtkWidget* w) const
 	g_object_set_data (G_OBJECT (item), "object", (void *) this);
 	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 	g_signal_connect_swapped (G_OBJECT (item), "changed", G_CALLBACK (on_fragment_changed), (void *) this);
-	g_signal_connect_swapped (G_OBJECT (item), "sel-changed", G_CALLBACK (on_fragment_sel_changed), (void *) this);
+	g_signal_connect_swapped (G_OBJECT (item), "sel-changed", G_CALLBACK (on_fragment_sel_changed), (void *) this);*/
 	/* add charge */
-	int charge = m_Atom->GetCharge ();
+	/*int charge = m_Atom->GetCharge ();
 	if (charge) {
 		double x, y, Angle, Dist;
 		unsigned char Pos = m_Atom->Atom::GetChargePosition (&Angle, &Dist);
@@ -442,18 +510,16 @@ void Fragment::Add (GtkWidget* w) const
 	}
 	pData->Items[this] = group;
 	const_cast <FragmentAtom *> (m_Atom)->DoBuildSymbolGeometry (pData->m_View);
-}
+}*/
 
-void Fragment::SetSelected (GtkWidget* w, int state)
+void Fragment::SetSelected (int state)
 {
-	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
-	GnomeCanvasGroup* group = pData->Items[this];
-	gchar const *chargecolor, *color;
+	GOColor chargecolor, color;
 	bool visible = true;
 	switch (state) {	
 	case SelStateUnselected:
-		color = NULL;
-		chargecolor = "black";
+		color = 0;
+		chargecolor = RGBA_BLACK;
 		visible = false;
 		break;
 	case SelStateSelected:
@@ -466,25 +532,26 @@ void Fragment::SetSelected (GtkWidget* w, int state)
 		chargecolor = color = DeleteColor;
 		break;
 	default:
-		color = NULL;
-		chargecolor = "black";
+		color = 0;
+		chargecolor = RGBA_BLACK;
 		break;
 	}
-	gpointer item;
-	item = g_object_get_data (G_OBJECT (group), "rect");
-	g_object_set (G_OBJECT (item),
-				"fill_color", color, NULL);
-	if (visible)
-		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
-	else
-		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
-	if ((item = g_object_get_data (G_OBJECT (group), "circle")))
-		g_object_set (item, "outline_color", chargecolor, NULL);
-	if ((item = g_object_get_data (G_OBJECT (group), "sign")))
-		g_object_set (item, "outline_color", chargecolor, NULL);
+	gccv::Group *group = static_cast <gccv::Group *> (m_Item);
+	std::list<gccv::Item *>::iterator it;
+	gccv::Item *item = group->GetFirstChild (it);
+	while (item) {
+		gccv::FillItem *fill;
+		if ((fill = dynamic_cast <gccv::Rectangle *> (item)))
+			fill->SetFillColor (color);
+		else if ((fill = dynamic_cast <gccv::FillItem *> (item)))
+			fill->SetFillColor (chargecolor);
+		else
+			static_cast <gccv::LineItem *> (item)->SetLineColor (chargecolor);
+		item = group->GetNextChild (it);
+	}
 }
 
-void Fragment::Update (GtkWidget* w) const
+/*void Fragment::Update (GtkWidget* w) const
 {
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
@@ -642,14 +709,13 @@ void Fragment::Update (GtkWidget* w) const
 		if (item) {
 			gtk_object_destroy (GTK_OBJECT (item));
 			g_object_set_data ((GObject*) group, "charge", NULL);
-		}
+	}
 	}
 	const_cast <FragmentAtom *> (m_Atom)->DoBuildSymbolGeometry (pData->m_View);
-}
+}*/
 
 xmlNodePtr Fragment::Save (xmlDocPtr xml) const
 {
-	const_cast <Fragment *> (this)->m_buf = pango_layout_get_text (m_Layout);
 	if (m_RealSave && !const_cast <Fragment *> (this)->Validate ())
 		return NULL;
 	xmlNodePtr node = xmlNewDocNode (xml, NULL, (xmlChar*) "fragment", NULL);
@@ -678,78 +744,76 @@ xmlNodePtr Fragment::Save (xmlDocPtr xml) const
 	return (SaveNode (xml, node))? node: NULL;
 }
 
-struct FilterStruct {
-	unsigned start, end;
-	list<PangoAttribute*> pal;
-};
-
-bool filter_func (PangoAttribute *attribute, struct FilterStruct *s)
-{
-	if (attribute->klass->type == PANGO_ATTR_RISE && ((PangoAttrInt*) attribute)->value > 0 &&
-		s->start <= attribute->start_index && s->end >= attribute->end_index) {
-		list<PangoAttribute*>::iterator i, iend = s->pal.end ();
-		for (i = s->pal.begin (); i != iend; i++)
-			if ((*i)->start_index > attribute->end_index)
-				break;
-		s->pal.insert (i, attribute);
-	}
-	return FALSE;
-}
-
 bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsigned end) const
 {
+	std::list <gccv::TextTag *> const *tags;
+	if (m_Item)
+		tags = m_TextItem->GetTags ();
+	else
+		tags = &m_TagList;
+	std::list <gccv::TextTag *>::const_iterator j, jend = tags->end ();
 	xmlNodePtr child;
-	struct FilterStruct s;
-	s.start = start;
-	s.end = end;
-	int charge;
-	if (m_AttrList == NULL)
-		const_cast <Fragment *> (this)->m_AttrList = pango_layout_get_attributes (m_Layout);
-	pango_attr_list_filter (m_AttrList, (PangoAttrFilterFunc) filter_func, &s);
-	list<PangoAttribute*>::iterator i, iend = s.pal.end ();
-	string str;
 	char *err;
-	for (i = s.pal.begin (); i != iend; i++) {
-		if (start < (*i)->start_index) {
-			str.assign (m_buf, start, (*i)->start_index - start);
-			xmlNodeAddContent (node, (const xmlChar*) str.c_str ());
-		}
-		str.assign (m_buf, (*i)->start_index, (*i)->end_index - (*i)->start_index);
-		child = xmlNewDocNode (xml, NULL, (xmlChar*) "charge", NULL);
-		if (!child)
-			return false;
-		charge = strtol (str.c_str (), &err, 10);
-		if (err && strcmp (err, "+") && strcmp (err, "-")) {
-			if (!m_RealSave) {
-				return  false;
-				xmlFreeNode (child);
+	int charge;
+	for (j = tags->begin (); j != jend; j++) {
+		if (end <= (*j)->GetStartIndex ())
+			continue;
+		if (start < (*j)->GetStartIndex ()) 
+			xmlNodeAddContentLen (node, reinterpret_cast <xmlChar const *> (m_buf.c_str () + start), (*j)->GetStartIndex () - start);
+		gccv::Tag tag = (*j)->GetTag ();
+		if (tag == gccv::Position) {
+			bool stacked;
+			double size;
+			gccv::TextPosition pos = static_cast <gccv::PositionTextTag *> (*j)->GetPosition (stacked, size);
+			switch (pos) {
+			case gccv::Subscript:
+				child = xmlNewDocNode (xml, NULL, reinterpret_cast <xmlChar const *> ("sub"), NULL);
+				break;
+			case gccv::Superscript:
+				child = xmlNewDocNode (xml, NULL, reinterpret_cast <xmlChar const *> ("sup"), NULL);
+				break;
+			default:
+				break;
+			}
+			xmlNodeAddContentLen ((child)? child: node, reinterpret_cast <xmlChar const *> (m_buf.c_str () + (*j)->GetStartIndex ()), (*j)->GetEndIndex () - (*j)->GetStartIndex ());
+		} else if (tag == ChargeTag) {
+			child = xmlNewDocNode (xml, NULL, reinterpret_cast <xmlChar const *> ("charge"), NULL);
+			charge = strtol (m_buf.c_str () + (*j)->GetStartIndex (), &err, 10);
+			if (charge == 0) {
+				if (*err == '+')
+					xmlNewProp (child, reinterpret_cast <xmlChar const *> ("value"), reinterpret_cast <xmlChar const *> ("1"));
+				else if (!strncmp (err, "−", strlen ("−")))
+					xmlNewProp (child, reinterpret_cast <xmlChar const *> ("value"), reinterpret_cast <xmlChar const *> ("-1"));
+				else
+					xmlNodeAddContentLen (child, reinterpret_cast <xmlChar const *> (m_buf.c_str () + (*j)->GetStartIndex ()), (*j)->GetEndIndex () - (*j)->GetStartIndex ());
+			} else {
+				if (*err != '+')
+					charge = -charge;
+				char *buf = g_strdup_printf ("%d", charge);
+				xmlNewProp (child, reinterpret_cast <xmlChar const *> ("value"), reinterpret_cast <xmlChar const *> (buf));
+				g_free (buf);
+			}
+		} else if (tag == StoichiometryTag) {
+			child = xmlNewDocNode (xml, NULL, reinterpret_cast <xmlChar const *> ("stoichiometry"), NULL);
+			// using the charge variable
+			charge = strtol (m_buf.c_str () + (*j)->GetStartIndex (), &err, 10);
+			if (charge <= 0)
+				xmlNodeAddContentLen (child, reinterpret_cast <xmlChar const *> (m_buf.c_str () + (*j)->GetStartIndex ()), (*j)->GetEndIndex () - (*j)->GetStartIndex ());
+			else {
+				char *buf = g_strdup_printf ("%d", charge);
+				xmlNewProp (child, reinterpret_cast <xmlChar const *> ("value"), reinterpret_cast <xmlChar const *> (buf));
+				g_free (buf);
 			}
-			Document *pDoc = (Document*) GetDocument ();
-			GtkWidget* w = gtk_message_dialog_new (
-											pDoc->GetWindow ()->GetWindow (),
-											GTK_DIALOG_DESTROY_WITH_PARENT,
-											GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-											_("Invalid charge."));
-			gtk_window_set_icon_name (GTK_WINDOW (w), "gchempaint");
-			gtk_dialog_run (GTK_DIALOG (w));
-			gtk_widget_destroy (w);
-			return false;
 		} else {
-			if (!charge)
-				charge = 1;
-			if (*err == '-')
-				charge = - charge;
-			char *buf = g_strdup_printf ("%d", charge);
-			xmlNewProp (child, (xmlChar*) "value", (xmlChar*) buf);
-			g_free (buf);
-			xmlAddChild (node, child);
+			xmlNodeAddContentLen (node, reinterpret_cast <xmlChar const *> (m_buf.c_str () + (*j)->GetStartIndex ()), (*j)->GetEndIndex () - (*j)->GetStartIndex ());
+			child = NULL;
 		}
-		start = (*i)->end_index;
-	}
-	if (start < end) {
-		str.assign (m_buf, start, end - start);
-		xmlNodeAddContent (node, (const xmlChar*) str.c_str ());
+		if (child)
+			xmlAddChild (node, child);
+		start = (*j)->GetEndIndex ();
 	}
+	if (start < end)
+		xmlNodeAddContentLen (node, reinterpret_cast <xmlChar const *> (m_buf.c_str () + start), end - start);
 	return true;
 }
 
@@ -768,15 +832,12 @@ bool Fragment::Load (xmlNodePtr node)
 	Theme *pTheme = pDoc->GetTheme ();
 	if (!TextObject::Load (node))
 		return false;
-	if (m_AttrList != NULL)
-		pango_attr_list_unref (m_AttrList);
-	m_AttrList = pango_attr_list_new ();
 	m_bLoading = true;
 	m_buf.clear (); // just in case
 	xmlNodePtr child = node->children;
 	char* tmp;
-	PangoAttribute *attr;
-	int size = pTheme->GetFontSize ();
+	double size = (double) pTheme->GetFontSize () / PANGO_SCALE;
+	gccv::TextTag *tag;
 	while (child) {
 		if (!strcmp ((const char*) child->name, "text")) {
 			tmp = (char*) xmlNodeGetContent (child);
@@ -815,34 +876,61 @@ bool Fragment::Load (xmlNodePtr node)
 			m_Atom->SetCoords (m_x, m_y);
 			m_EndAtom = m_buf.length ();
 		} else if (!strcmp ((const char*) child->name, "charge")) {
-			int start = m_buf.length (), end;
+			int start = m_buf.length ();
 			tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
-			int charge = atoi (tmp);
-			xmlFree (tmp);
-			if (abs(charge) > 1)
-				tmp = g_strdup_printf ("%d%c", abs (charge), (charge > 0)? '+': '-');
-			else if (charge == 1)
-				tmp = g_strdup ("+");
-			else if (charge == -1)
-				tmp = g_strdup ("-");
-			else
-				tmp = g_strdup ("");//should not occur!
-			m_buf += tmp;
-			end = m_buf.length ();
-			attr = pango_attr_size_new (size * 2 / 3);
-			attr->start_index = start;
-			attr->end_index = end;
-			pango_attr_list_insert (m_AttrList, attr);
-			attr = pango_attr_rise_new (2 * size / 3);
-			attr->start_index = start;
-			attr->end_index = end;
-			pango_attr_list_insert (m_AttrList, attr);
+			if (tmp) {
+				int charge = atoi (tmp);
+				xmlFree (tmp);
+				if (abs(charge) > 1)
+					tmp = g_strdup_printf ("%d%s", abs (charge), (charge > 0)? "+": "−");
+				else if (charge == 1)
+					tmp = g_strdup ("+");
+				else if (charge == -1)
+					tmp = g_strdup ("−");
+				else
+					tmp = g_strdup ("");//should not occur!
+				m_buf += tmp;
+				g_free (tmp);
+			} else { // allow things such as "z+" or "δ−"
+				tmp = reinterpret_cast <char *> (xmlNodeGetContent (child));
+				m_buf += tmp;
+				xmlFree (tmp);
+			}
+			tag = new ChargeTextTag (size);
+			tag->SetStartIndex (start);
+			tag->SetEndIndex (m_buf.length ());
+			m_TagList.push_back (tag);
+		} else if (!strcmp ((const char*) child->name, "stoichiometry")) {
+			int start = m_buf.length ();
+			tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
+			if (tmp) {
+				int nb = atoi (tmp);
+				xmlFree (tmp);
+				if (nb <=0)
+					return false; // negative number is not allowed
+				tmp = g_strdup_printf ("%u", nb);
+				m_buf += tmp;
+				g_free (tmp);
+			} else { // allow things such as "n"
+				tmp = reinterpret_cast <char *> (xmlNodeGetContent (child));
+				m_buf += tmp;
+				xmlFree (tmp);
+			}
+			tag = new StoichiometryTextTag (size);
+			tag->SetStartIndex (start);
+			tag->SetEndIndex (m_buf.length ());
+			m_TagList.push_back (tag);
+		} else if (!strcmp ((const char*) child->name, "sub")) {
+		} else if (!strcmp ((const char*) child->name, "sup")) {
 		}
 		child = child->next;
 	}
-	if (m_Layout) {
-		pango_layout_set_text (m_Layout, m_buf.c_str (), -1);
-		pango_layout_set_attributes (m_Layout, m_AttrList);
+	if (m_TextItem) {
+		m_TextItem->SetText (m_buf);
+		while (!m_TagList.empty ()) {
+			m_TextItem->InsertTextTag (m_TagList.front ());
+			m_TagList.pop_front ();
+		}
 	}
 	AnalContent ();
 	m_bLoading = false;
@@ -853,8 +941,8 @@ void Fragment::AnalContent ()
 {
 	if (!m_Atom->GetParent ())
 		AddChild (m_Atom);
-	unsigned end = (m_Layout)? strlen (pango_layout_get_text (m_Layout)): m_buf.length ();
-	AnalContent(0, end);
+	unsigned end = m_buf.length ();
+	AnalContent (0, end);
 }
 
 typedef struct
@@ -881,47 +969,59 @@ void Fragment::AnalContent (unsigned start, unsigned &end)
 	if (!pDoc)
 		return;
 	Theme *pTheme = pDoc->GetTheme ();
-	char const *text;
-	PangoAttrList *l;
-	if (m_Layout) {
-		text = pango_layout_get_text (m_Layout);
-		l = pango_layout_get_attributes (m_Layout);
-	} else {
-		text = m_buf.c_str ();
-		l = pango_attr_list_ref (m_AttrList);
-	}
-	bool Charge = false;
-	unsigned start_tag, end_tag, next;
-	char c;
-	ChargeFindStruct s;
-	s.index = s.end = 0;
-	if (start > 0) {
-		// search if a charge is at preceeding character
-		s.result = false;
-		s.index = start;
-		pango_attr_list_filter (l, (PangoAttrFilterFunc) search_for_charge, &s);
-		Charge = s.result;
-	} else if (text[start] == '+' || text[start] == '-')
+	char const *text = m_buf.c_str ();
+	list <gccv::TextTag *> const *tags = (m_TextItem)? m_TextItem->GetTags (): &m_TagList;
+	list <gccv::TextTag *>::const_iterator i, iend = tags->end ();
+	bool Charge = false, Stoich = false;
+	for (i = tags->begin (); i != iend; i++)
+		if ((*i)->GetTag () == ChargeTag && (*i)->GetStartIndex () < start && (*i)->GetEndIndex () >= start) {
+			Charge = true; 
+			break;
+		}
+	if (m_Mode == AutoMode && !Charge && (text[start] == '+' || !strncmp (text + start, "−", strlen ("−"))))
 		Charge = true;
-	else
-		Charge = false;
+	char c;
+	gccv::TextTag *tag;
+	unsigned start_tag, end_tag, next;
 	next = start;
-	start_tag = end_tag = start;
+/*	start_tag = end_tag = start;*/
 	while (start < end) {
 		c = text[start];
 		if ((c >= '0') && (c <= '9')) {
-			s.result = false;
-			s.index = start;
-			pango_attr_list_filter (l, (PangoAttrFilterFunc) search_for_charge, &s);
-			Charge = s.result;
-			next = start + 1;
-			// add new size and rise attributes
-			int size = pTheme->GetFontSize ();
-			PangoAttribute *attr = pango_attr_size_new (size * 2 / 3);
+			tag = NULL;
+			Charge = Stoich = false;
+			for (i = tags->begin (); i != iend; i++)
+				if ((*i)->GetTag () == ChargeTag && (*i)->GetStartIndex () < start && (*i)->GetEndIndex () >= start) {
+					Charge = true; 
+					break;
+				} else if ((*i)->GetTag () == StoichiometryTag && (*i)->GetStartIndex () <= start && (*i)->GetEndIndex () >= start) {
+					Stoich = true; 
+					break;
+				}
+			next = start + 1; // a figure is a one byte character
+			// add new tag
+			double size = (double) pTheme->GetFontSize () / PANGO_SCALE;
+			if (!Charge) {
+				if (!Stoich) {
+					tag = new StoichiometryTextTag (size);
+					tag->SetStartIndex (start);
+					tag->SetEndIndex (next);
+				}
+			} else {
+				tag = new ChargeTextTag (size);
+				tag->SetStartIndex (start);
+				tag->SetEndIndex (next);
+			}
+			if (tag) {
+				if (m_TextItem)
+					m_TextItem->InsertTextTag (tag);
+				else
+					m_TagList.push_back (tag);
+			}
+/*			PangoAttribute *attr = pango_attr_size_new (size * 2 / 3);
 			attr->start_index = start;
 			attr->end_index = next;
 			pango_attr_list_change (l, attr);
-			if (!Charge)
 				attr = pango_attr_rise_new (-size / 3);
 			else {
 				if (text[start - 1] == '+' || text[start - 1] == '-') {
@@ -940,10 +1040,10 @@ void Fragment::AnalContent (unsigned start, unsigned &end)
 			}
 			attr->start_index = start;
 			attr->end_index = next;
-				pango_attr_list_change (l, attr);
+				pango_attr_list_change (l, attr);*/
 			start = next - 1;
 		} else if ((c == '+') || (c == '-')) {
-			if (!m_bLoading) {
+			/*if (!m_bLoading) {
 				//do not allow both local and global charges
 				if (m_Atom->GetCharge ())
 					m_Atom->SetCharge (0);
@@ -998,19 +1098,19 @@ void Fragment::AnalContent (unsigned start, unsigned &end)
 						pango_attr_list_insert (l, attr);
 						attr = pango_attr_rise_new (2 * size / 3);
 						pango_attr_list_insert (l, attr);
-						gcp_pango_layout_replace_text (m_Layout, s.index, s.end - s.index + 1, buf, l);
+//						gcp_pango_layout_replace_text (m_Layout, s.index, s.end - s.index + 1, buf, l);
 						pango_attr_list_unref (l);
 						m_StartSel = m_EndSel = s.index + strlen (buf);
-						end += m_StartSel - start - 1;
-						GnomeCanvasPango* text = pDoc->GetView ()->GetActiveRichText ();
-						gnome_canvas_pango_set_selection_bounds (text, m_StartSel, m_EndSel);
-						g_free (buf);
+						end += m_StartSel - start - 1;*/
+/*						GnomeCanvasPango* text = pDoc->GetView ()->GetActiveRichText ();
+						gnome_canvas_pango_set_selection_bounds (text, m_StartSel, m_EndSel);*/
+/*						g_free (buf);
 					}
 				}
-			}
+			}*/
 		} else
 			Charge = false;
-		start++;
+		start = g_utf8_find_next_char (m_buf.c_str () + start, NULL) - m_buf.c_str ();
 	}
 }
 
@@ -1018,7 +1118,7 @@ void Fragment::AnalContent (unsigned start, unsigned &end)
 Must return NULL if active tool is FragmentTool because this tool needs a fragment, not an atom
 TODO: use x and y to figure the best atom in the fragment
 */
-Object* Fragment::GetAtomAt (double x, double y, double z)
+Object* Fragment::GetAtomAt (double x, double y, G_GNUC_UNUSED double z)
 {
 	Document* pDoc = (Document*) GetDocument ();
 	Theme *pTheme = pDoc->GetTheme ();
@@ -1034,15 +1134,16 @@ Object* Fragment::GetAtomAt (double x, double y, double z)
 	y0 = (y - m_y) * pTheme->GetZoomFactor () + m_ascent;
 	if ((x0 < 0.) || (x0 > m_length) || (y0 < 0.) || (y0 > m_height))
 		return NULL;
-	int index, cur, trailing;
-	pango_layout_xy_to_index (m_Layout, (int) (x0 * PANGO_SCALE), (int) (y0 * PANGO_SCALE), &index, &trailing);
+	unsigned index, trailing;
+	int cur;
+	index = m_TextItem->GetIndexAt (x0, y0);
 	char c = m_buf[index];
 	cur = index;
-	while ((c >= 'a') && (c <= 'z') && cur >=0) {
+	while ((c >= 'a') && (c <= 'z') && cur >= 0) {
 		cur--;
 		c = m_buf[cur];
 	}
-	if (index - cur > static_cast <int> (Residue::MaxSymbolLength))
+	if (index - cur > Residue::MaxSymbolLength)
 		cur = index - Residue::MaxSymbolLength;
 	if (cur < 0)
 		cur = 0;
@@ -1078,12 +1179,12 @@ Object* Fragment::GetAtomAt (double x, double y, double z)
 			delete pOldAtom;
 			AddChild (m_Atom);
 		}
-		m_x -= m_lbearing / pTheme->GetZoomFactor () ;
-		PangoRectangle rect;
-		pango_layout_index_to_pos (m_Layout, index, &rect);
-		m_lbearing = rect.x / PANGO_SCALE;
-		pango_layout_index_to_pos (m_Layout, index + i, &rect);
-		m_lbearing += rect.x / PANGO_SCALE;
+		m_x -= m_lbearing / pTheme->GetZoomFactor ();
+		gccv::Rect rect;
+		m_TextItem->GetPositionAtIndex (index, rect);
+		m_lbearing = rect.x0;
+		m_TextItem->GetPositionAtIndex (index + i, rect);
+		m_lbearing += rect.x0;
 		m_lbearing /=  2;
 		m_x += m_lbearing / pTheme->GetZoomFactor ();
 		m_Atom->SetCoords(m_x, m_y);
@@ -1118,11 +1219,11 @@ Object* Fragment::GetAtomAt (double x, double y, double z)
 		m_BeginAtom = index;
 		m_EndAtom = trailing;
 		m_x -= m_lbearing / pTheme->GetZoomFactor () ;
-		PangoRectangle rect;
-		pango_layout_index_to_pos (m_Layout, index, &rect);
-		m_lbearing = rect.x / PANGO_SCALE;
-		pango_layout_index_to_pos (m_Layout, trailing, &rect);
-		m_lbearing += rect.x / PANGO_SCALE;
+		gccv::Rect rect;
+		m_TextItem->GetPositionAtIndex (index, rect);
+		m_lbearing = rect.x0;
+		m_TextItem->GetPositionAtIndex (trailing, rect);
+		m_lbearing += rect.x0;
 		m_lbearing /=  2;
 		m_x += m_lbearing / pTheme->GetZoomFactor ();
 		m_Atom->SetCoords(m_x, m_y);
@@ -1142,11 +1243,11 @@ void Fragment::OnChangeAtom ()
 	if (m_bLoading)
 		return;
 	Document *pDoc = (Document*) GetDocument ();
-	if (!pDoc || !m_Atom)
-		return;
-	char const *sym = m_Atom->GetSymbol ();
-	gcp_pango_layout_replace_text (m_Layout, m_BeginAtom, m_EndAtom - m_BeginAtom, sym, pDoc->GetPangoAttrList ());
-	m_EndAtom = m_BeginAtom + strlen (sym);
+	if (!pDoc) return;
+	string sym = m_Atom->GetSymbol ();
+	m_TextItem->ReplaceText (sym, m_BeginAtom, m_EndAtom - m_BeginAtom);
+	m_EndAtom = m_BeginAtom + sym.length ();
+	// FIXME: we probably need to insert a tag there
 	OnChanged (false);
 }
 
@@ -1155,7 +1256,7 @@ int Fragment::GetElementAtPos (unsigned start, unsigned &end)
 	int Z;
 	char text[4];
 	memset (text, 0, 4);
-	strncpy (text, ((m_Layout)?pango_layout_get_text ( m_Layout): m_buf.c_str ()) + start, 3);
+	strncpy (text, m_buf.c_str () + start, 3);
 	for (unsigned i = strlen (text); i > 0; i--) {
 		text[i] = 0;
 		if ((Z = Element::Z (text))) {
@@ -1176,7 +1277,7 @@ int Fragment::GetChargePosition (FragmentAtom *pAtom, unsigned char &Pos, double
 	if (!pDoc)
 		return 0;
 	GtkWidget* pWidget = pDoc->GetView ()->GetWidget ();
-	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (pWidget), "data");
+/*	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (pWidget), "data");
 	GnomeCanvasGroup *item = pData->Items[this];
 	if (!item)
 		return 0;
@@ -1277,7 +1378,7 @@ int Fragment::GetChargePosition (FragmentAtom *pAtom, unsigned char &Pos, double
 		x = m_x - width / 2.0;
 		y = m_y;
 		return -1;
-	}
+	}*/
 	return 0;
 }
 
@@ -1290,11 +1391,7 @@ bool Fragment::Validate ()
 		return true;
 	if (m_Atom->GetZ() == 0 || (dynamic_cast <FragmentResidue*> (m_Atom) && !((FragmentResidue*) m_Atom)->GetResidue ())) {
 		Document *pDoc = dynamic_cast<Document*> (GetDocument ());
-		GtkWidget* pWidget = pDoc->GetView ()->GetWidget ();
-		WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (pWidget), "data");
-		GnomeCanvasGroup *item = pData->Items[this];
-		GnomeCanvasPango* text = GNOME_CANVAS_PANGO (g_object_get_data (G_OBJECT (item), "fragment"));
-		gnome_canvas_pango_set_selection_bounds (text, m_BeginAtom, (m_EndAtom == m_BeginAtom)? m_EndAtom + 1: m_EndAtom);
+		m_TextItem->SetSelectionBounds (m_BeginAtom, (m_EndAtom == m_BeginAtom)? m_EndAtom + 1: m_EndAtom);
 		GtkWidget* w = gtk_message_dialog_new (
 										GTK_WINDOW (pDoc->GetWindow ()->GetWindow ()),
 										GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -1304,24 +1401,23 @@ bool Fragment::Validate ()
 		gtk_widget_destroy (w);
 		return false;
 	}
-	//now scan for charges and validate
-	struct FilterStruct s;
-	s.start = 0;
-	s.end = m_buf.length ();
-	if (m_AttrList == NULL)
+	//now scan for charges and stoichiometric numbers and validate
+	gccv::TextTagList chargetags;
+	list <gccv::TextTag *> const *tags = m_TextItem->GetTags ();
+	// we get everything displayed as superscript and see if it is a charge
+	// all text not recognized as a charge will be analyzed through the gcu::Formula mechanism
+	// FIXME: write that code
+
+/*	if (m_AttrList == NULL)
 		m_AttrList = pango_layout_get_attributes (m_Layout);
-	pango_attr_list_filter (m_AttrList, (PangoAttrFilterFunc) filter_func, &s);
-	list<PangoAttribute*>::iterator i, iend = s.pal.end ();
+	pango_attr_list_filter (m_AttrList, (PangoAttrFilterFunc) filter_func, &s);*/
+/*	list<PangoAttribute*>::iterator i, iend = s.pal.end ();
 	for (i = s.pal.begin (); i != iend; i++) {
 		charge = m_buf.c_str () + (*i)->start_index;
 		strtol (charge, &err, 10);
 		if (*err != '+' && *err != '-' && err - m_buf.c_str () != (int) (*i)->end_index) {
 			Document *pDoc = dynamic_cast<Document*> (GetDocument ());
-			GtkWidget* pWidget = pDoc->GetView ()->GetWidget ();
-			WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (pWidget), "data");
-			GnomeCanvasGroup *item = pData->Items[this];
-			GnomeCanvasPango* text = GNOME_CANVAS_PANGO (g_object_get_data (G_OBJECT (item), "fragment"));
-			gnome_canvas_pango_set_selection_bounds (text, (*i)->start_index, (*i)->end_index);
+			text->SetSelectionBounds (m_BeginAtom, (m_EndAtom == m_BeginAtom)? m_EndAtom + 1: m_EndAtom);
 			GtkWidget* w = gtk_message_dialog_new (
 											GTK_WINDOW (pDoc->GetWindow ()->GetWindow ()),
 											GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -1331,7 +1427,7 @@ bool Fragment::Validate ()
 			gtk_widget_destroy(w);
 			return false;
 		}
-	}
+	}*/
 	return true;
 }
 
@@ -1350,12 +1446,12 @@ double Fragment::GetYAlign ()
 	return m_y;
 }
 
-int Fragment::GetAvailablePosition (double& x, double& y)
+int Fragment::GetAvailablePosition (G_GNUC_UNUSED double& x, G_GNUC_UNUSED double& y)
 {
 	return 0;
 }
 
-bool Fragment::GetPosition (double angle, double& x, double& y)
+bool Fragment::GetPosition (G_GNUC_UNUSED double angle, G_GNUC_UNUSED double& x, G_GNUC_UNUSED double& y)
 {
 	return false;
 }
@@ -1468,9 +1564,6 @@ bool Fragment::Analyze () {
 		m_Inversable = true;
 	}
 //	int valence = m_Atom->GetValence ();
-	if (m_AttrList != NULL)
-		pango_attr_list_unref (m_AttrList);
-	m_AttrList = pango_attr_list_new ();
 	AnalContent ();
 	return true;
 }
@@ -1492,9 +1585,6 @@ void Fragment::Update () {
 			delete formula;
 			m_EndAtom = m_buf.length ();
 			m_BeginAtom = m_EndAtom - strlen (m_Atom->GetSymbol ());
-			if (m_AttrList != NULL)
-				pango_attr_list_unref (m_AttrList);
-			m_AttrList = pango_attr_list_new ();
 			AnalContent ();
 		} else if (angle > 91. || angle < -91.) {
 		}
