@@ -4,7 +4,7 @@
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,22 +23,26 @@
  */
 
 #include "config.h"
-#include "widgetdata.h"
-#include "view.h"
-#include "settings.h"
-#include "document.h"
 #include "application.h"
-#include "tool.h"
-#include "tools.h"
+#include "atom.h"
+#include "bond.h"
+#include "document.h"
+#include "settings.h"
 #include "text.h"
 #include "theme.h"
+#include "tool.h"
+#include "tools.h"
+#include "view.h"
+#include "widgetdata.h"
 #include "window.h"
-#include <canvas/gprintable.h>
-#include <canvas/gcp-canvas-group.h>
-#include <canvas/gcp-canvas-rect-ellipse.h>
+#include <gccv/canvas.h>
+#include <gccv/group.h>
+#include <gccv/text.h>
 #include <gsf/gsf-output-gio.h>
+#include <gsf/gsf-output-memory.h>
 #include <cairo-pdf.h>
 #include <cairo-ps.h>
+#include <cairo-svg.h>
 #include <pango/pango-context.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n-lib.h>
@@ -51,120 +55,16 @@
 #include <iostream>
 #include <unistd.h>
 
-/*
-Derivation of a new widget from gnome_canvas with an event for updating canvas size
-*/
-
-typedef struct _GnomeCanvasGCP           GnomeCanvasGCP;
-typedef struct _GnomeCanvasGCPClass      GnomeCanvasGCPClass;
-
-#define GNOME_TYPE_CANVAS_GCP            (gnome_canvas_gcp_get_type ())
-#define GNOME_CANVAS_GCP(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCP))
-#define GNOME_CANVAS_CLASS_GCP(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCPClass))
-#define GNOME_IS_CANVAS_GCP(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOME_TYPE_CANVAS_GCP))
-#define GNOME_IS_CANVAS_GCP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOME_TYPE_CANVAS_GCP))
-#define GNOME_CANVAS_GCP_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCPClass))
-
-enum {
-  UPDATE_BOUNDS,
-  LAST_SIGNAL
-};
-
-struct _GnomeCanvasGCP {
-	GnomeCanvas canvas;
-};
-
-struct _GnomeCanvasGCPClass {
-	GnomeCanvasClass parent_class;
-
-	void (* update_bounds) (GnomeCanvasGCP *canvas);
-};
-
-GType gnome_canvas_gcp_get_type (void) G_GNUC_CONST;
-static void gnome_canvas_gcp_class_init (GnomeCanvasGCPClass *Class);
-static void gnome_canvas_gcp_init (GnomeCanvasGCP *canvas);
-static void gnome_canvas_gcp_update_bounds (GnomeCanvasGCP *canvas);
-static GtkWidget *gnome_canvas_gcp_new (void);
-
-static guint gnome_canvas_gcp_signals[LAST_SIGNAL] = { 0 };
-
-GType
-gnome_canvas_gcp_get_type (void)
-{
-	static GType canvas_gcp_type;
-
-	if (!canvas_gcp_type) {
-		static const GTypeInfo object_info = {
-			sizeof (GnomeCanvasGCPClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) gnome_canvas_gcp_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,			/* class_data */
-			sizeof (GnomeCanvasGCP),
-			0,			/* n_preallocs */
-			(GInstanceInitFunc) gnome_canvas_gcp_init,
-			NULL			/* value_table */
-		};
-
-		canvas_gcp_type = g_type_register_static (GNOME_TYPE_CANVAS, "GnomeCanvasGCP",
-						      &object_info, (GTypeFlags) 0);
-	}
-
-	return canvas_gcp_type;
-}
-
-GtkWidget *gnome_canvas_gcp_new (void)
-{
-	return GTK_WIDGET (g_object_new (GNOME_TYPE_CANVAS_GCP, "aa", TRUE, NULL));
-}
-
-void gnome_canvas_gcp_class_init (GnomeCanvasGCPClass *Class)
-{
-	GObjectClass *gobject_class = G_OBJECT_CLASS (Class);
-	gnome_canvas_gcp_signals[UPDATE_BOUNDS] =
-	g_signal_new ("update_bounds",
-				  G_TYPE_FROM_CLASS (gobject_class),
-				  G_SIGNAL_RUN_LAST,
-				  G_STRUCT_OFFSET (GnomeCanvasGCPClass, update_bounds),
-				  NULL, NULL,
-				  g_cclosure_marshal_VOID__VOID,
-				  G_TYPE_NONE, 0
-				  );
-	Class->update_bounds = gnome_canvas_gcp_update_bounds;
-}
-
-void gnome_canvas_gcp_init (GnomeCanvasGCP *canvas)
-{
-}
-
-void gnome_canvas_gcp_update_bounds (GnomeCanvasGCP *canvas)
-{
-	GnomeCanvas* w = GNOME_CANVAS (canvas);
-	while (w->idle_id)
-		gtk_main_iteration();
-	gnome_canvas_update_now (w);
-	gcp::WidgetData* pData = (gcp::WidgetData*) g_object_get_data (G_OBJECT (canvas), "data");
-	double x1, y1, x2, y2;
-	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Group), &x1, &y1, &x2, &y2);
-	if (x1 > x2)
-		x1 = x2 = 0.;
-	if (y1 > y2)
-		y1 = y2 = 0.;
-	gcp::View *pView = (gcp::View*) g_object_get_data (G_OBJECT (canvas), "view");
-	pView->UpdateSize (x1, y1, x2, y2);
-}
-
 using namespace gcu;
 using namespace std;
 
 namespace gcp {
 
-bool on_event (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
+/* bool on_event (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 {
 	View* pView = (View*) g_object_get_data(G_OBJECT(widget), "view");
 	return pView->OnEvent(item, event, widget);
-}
+} */
 
 static bool on_destroy (GtkWidget *widget, View * pView)
 {
@@ -172,18 +72,13 @@ static bool on_destroy (GtkWidget *widget, View * pView)
 	return true;
 }
 
-static bool on_size (GtkWidget *widget, GtkAllocation *alloc, View * pView)
-{
-	pView->OnSize (widget, alloc->width, alloc->height);
-	return true;
-}
-
 void on_receive (GtkClipboard *clipboard, GtkSelectionData *selection_data, View * pView)
 {
 	pView->OnReceive (clipboard, selection_data);
 }
 
-View::View (Document *pDoc, bool Embedded)
+View::View (Document *pDoc, bool Embedded):
+	Client ()
 {
 	m_pDoc = pDoc;
 	Theme *pTheme = pDoc->GetTheme ();
@@ -210,14 +105,23 @@ View::View (Document *pDoc, bool Embedded)
 	m_UIManager = gtk_ui_manager_new ();
 	m_Dragging = false;
 	m_pWidget = NULL;
-	m_PangoContext = NULL;
 	m_CurObject = NULL;
+	m_CurAtom = NULL;
+	PangoLayout *layout = pango_layout_new (gccv::Text::GetContext ());
+	pango_layout_set_text (layout, "C", 1);
+	pango_layout_set_font_description (layout, m_PangoFontDesc);
+	PangoRectangle rect;
+	pango_layout_get_extents (layout, &rect, NULL);
+	m_CHeight =  double (rect.height) / PANGO_SCALE / 2.0;
+	m_BaseLineOffset =  m_CHeight / m_pDoc->GetTheme ()->GetZoomFactor ();
+	pango_layout_set_text (layout, "H", 1);
+	pango_layout_get_extents (layout, &rect, NULL);
+	m_HWidth = (double (rect.width) / 2.0 + rect.x) / PANGO_SCALE;
+	g_object_unref (layout);
 }
 
 View::~View ()
 {
-	if (m_PangoContext)
-		g_object_unref (G_OBJECT (m_PangoContext));
 	if (m_sFontName)
 		g_free (m_sFontName);
 	if (m_sSmallFontName)
@@ -225,9 +129,10 @@ View::~View ()
 	pango_font_description_free (m_PangoFontDesc);
 	pango_font_description_free (m_PangoSmallFontDesc);
 	g_object_unref (m_UIManager);
+	// we don't need to delete the canvas, since destroying the widget does the job.
 }
 
-bool View::OnEvent (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
+/*bool View::OnEvent (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 {
 	Application *App = m_pDoc->GetApplication ();
 	Theme *pTheme = m_pDoc->GetTheme ();
@@ -361,21 +266,34 @@ bool View::OnEvent (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 		break;
 	}
 	return false;
-}
+}*/
 
-void View::AddObject (Object const *pObject)
+void View::AddObject (Object *pObject)
 {
-	std::list<GtkWidget*>::iterator i, end = m_Widgets.end ();
-	for (i = m_Widgets.begin (); i != end; i++)
-		pObject->Add (*i);
+	gccv::ItemClient *client = dynamic_cast <gccv::ItemClient *> (pObject);
+	if (client)
+		client->AddItem ();
+	// now, add the children
+	map<string, Object*>::iterator i;
+	Object *child = pObject->GetFirstChild (i);
+	while (child) {
+		AddObject (child);
+		child = pObject->GetNextChild (i);
+	}
 }
 
 GtkWidget* View::CreateNewWidget ()
 {
-	gtk_widget_push_colormap (gdk_rgb_get_colormap ());
+	if (m_Canvas)
+		return m_Canvas->GetWidget ();
+/*	gtk_widget_push_colormap (gdk_rgb_get_colormap ());
 	m_pWidget = gnome_canvas_gcp_new();
-	gtk_widget_pop_colormap ();
-	GtkWidget* pWidget = (m_Widgets.size() > 0) ? m_Widgets.front () : NULL;
+	gtk_widget_pop_colormap ();*/
+	m_Canvas = new gccv::Canvas (this);
+	m_Canvas->SetBackgroundColor (RGBA_WHITE);
+	m_pWidget = m_Canvas->GetWidget ();
+	m_Canvas->SetGap (3.); // FIXME: make this configurable
+//	GtkWidget* pWidget = (m_Widgets.size() > 0) ? m_Widgets.front () : NULL;
 	if (m_pWidget) {
 		g_object_set_data (G_OBJECT (m_pWidget), "view", this);
 		g_object_set_data (G_OBJECT (m_pWidget), "doc", m_pDoc);
@@ -383,10 +301,10 @@ GtkWidget* View::CreateNewWidget ()
 		m_pData->Canvas = m_pWidget;
 		g_object_set_data (G_OBJECT (m_pWidget), "data", m_pData);
 		m_pData->m_View = this;
-		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), 1);
-		gnome_canvas_set_scroll_region (GNOME_CANVAS (m_pWidget), 0, 0, m_width, m_height);
+//		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), 1);
+//		gnome_canvas_set_scroll_region (GNOME_CANVAS (m_pWidget), 0, 0, m_width, m_height);
 		m_pData->Zoom = 1.0;
-		m_pData->Background = gnome_canvas_item_new (
+/*		m_pData->Background = gnome_canvas_item_new (
 									gnome_canvas_root (GNOME_CANVAS (m_pWidget)),
 									gnome_canvas_rect_ext_get_type (),
 									"x1", 0.0,
@@ -400,13 +318,11 @@ GtkWidget* View::CreateNewWidget ()
 									gnome_canvas_group_ext_get_type (),
 									NULL));
 		if (m_pDoc->GetEditable ())
-			g_signal_connect (G_OBJECT (m_pData->Background), "event", G_CALLBACK (on_event), m_pWidget);
+			g_signal_connect (G_OBJECT (m_pData->Background), "event", G_CALLBACK (on_event), m_pWidget);*/
 		g_signal_connect (G_OBJECT (m_pWidget), "destroy", G_CALLBACK (on_destroy), this);
-		g_signal_connect (G_OBJECT (m_pWidget), "size_allocate", G_CALLBACK (on_size), this);
-		g_signal_connect (G_OBJECT (m_pWidget), "realize", G_CALLBACK (gnome_canvas_gcp_update_bounds), this);
 		gtk_widget_show (m_pWidget);
-		m_Widgets.push_back (m_pWidget);
-		if (pWidget) {
+//		m_Widgets.push_back (m_pWidget);
+/*		if (pWidget) {
 			WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (pWidget), "data");
 			std::map<Object const*, GnomeCanvasGroup*>::iterator i, iend = pData->Items.end ();
 			for (i = pData->Items.begin (); i != iend; i++)
@@ -415,9 +331,7 @@ GtkWidget* View::CreateNewWidget ()
 			for (i = pData->Items.begin (); i != iend; i++)
 				if ((*i).first->GetType () == gcu::BondType)
 				 (*i).first->Add (m_pWidget);
-		} else {
-			m_PangoContext = gtk_widget_create_pango_context (m_pWidget);
-			g_object_ref (G_OBJECT (m_PangoContext));
+		} else*/ {
 			UpdateFont ();
 		}
 	}
@@ -433,7 +347,7 @@ void View::OnDestroy (GtkWidget* widget)
 		delete m_pDoc;
 }
 
-GnomeCanvasItem* View::GetCanvasItem (GtkWidget* widget, Object* Object)
+/*GnomeCanvasItem* View::GetCanvasItem (GtkWidget* widget, Object* Object)
 {
 	WidgetData* pData = reinterpret_cast<WidgetData*> (g_object_get_data (G_OBJECT (widget), "data"));
 	if ((!pData) || (pData->m_View != this))
@@ -442,18 +356,20 @@ GnomeCanvasItem* View::GetCanvasItem (GtkWidget* widget, Object* Object)
 	if (!result)
 		pData->Items.erase (Object);
 	return result;
-}
+}*/
 
-void View::Update (Object const *pObject)
+void View::Update (Object *pObject)
 {
-	std::list<GtkWidget*>::iterator i;
-	for (i = m_Widgets.begin (); i != m_Widgets.end (); i++)
-		pObject->Update(*i);
-}
-
-GnomeCanvasItem* View::GetBackground ()
-{
-	return m_pData->Background;
+	gccv::ItemClient *client = dynamic_cast <gccv::ItemClient *> (pObject);
+	if (client)
+		client->UpdateItem ();
+	// now, add the children
+	map<string, Object*>::iterator i;
+	Object *child = pObject->GetFirstChild (i);
+	while (child) {
+		Update (child);
+		child = pObject->GetNextChild (i);
+	}
 }
 
 double View::GetZoomFactor ()
@@ -463,34 +379,34 @@ double View::GetZoomFactor ()
 
 void View::UpdateFont ()
 {
-	pango_context_set_font_description (m_PangoContext, m_PangoFontDesc);
-	PangoLayout* pl = pango_layout_new (m_PangoContext);
+	PangoLayout* pl = pango_layout_new (gccv::Text::GetContext ());
+	pango_layout_set_font_description (pl, m_PangoFontDesc);
 	PangoRectangle rect;
 	pango_layout_set_text (pl, "lj", 2);
 	pango_layout_get_extents (pl, &rect, NULL);
 	m_dFontHeight = rect.height / PANGO_SCALE;
-	g_object_unref (G_OBJECT (pl));
-	pl = pango_layout_new (m_PangoContext);
 	pango_layout_set_text (pl, "C", 1);
 	pango_layout_get_extents (pl, &rect, NULL);
-	m_BaseLineOffset =  (double (rect.height / PANGO_SCALE) / 2.0) / m_pDoc->GetTheme ()->GetZoomFactor ();
+	m_CHeight = (double) rect.height / PANGO_SCALE / 2.0;
+	m_BaseLineOffset =  m_CHeight / m_pDoc->GetTheme ()->GetZoomFactor ();
+	pango_layout_set_text (pl, "H", 1);
+	pango_layout_get_extents (pl, &rect, NULL);
+	m_HWidth = (double (rect.width) / 2.0 + rect.x) / PANGO_SCALE;
 	g_object_unref (G_OBJECT (pl));
 }
 	
 void View::Remove (Object* pObject)
 {
-	std::list<GtkWidget*>::iterator i, end = m_Widgets.end ();
-	for (i = m_Widgets.begin (); i != end; i++) {
-		WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (*i), "data");
-		Object* pObj = pObject->GetMolecule ();
-		if (pObj)
-			pData->SelectedObjects.remove (pObj);
-		else
-			pData->SelectedObjects.remove (pObject);
-		if (pData->Items[pObject])
-			gtk_object_destroy (GTK_OBJECT (pData->Items[pObject]));
-		pData->Items.erase (pObject);
-	}
+	if (!m_pWidget)
+		return;
+	Object* pObj = pObject->GetMolecule ();
+	if (pObj)
+		m_pData->SelectedObjects.remove (pObj);
+	else
+		m_pData->SelectedObjects.remove (pObject);
+	gccv::ItemClient *client = dynamic_cast <gccv::ItemClient *> (pObject);
+	if (client && client->GetItem ())
+		delete client->GetItem ();
 }
 
 void View::OnDeleteSelection (GtkWidget* w)
@@ -501,14 +417,6 @@ void View::OnDeleteSelection (GtkWidget* w)
 	Object *parent;
 	if (!pActiveTool->DeleteSelection ()) {
 		m_pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
-		WidgetData *pData;
-		std::list<GtkWidget*>::iterator j, jend = m_Widgets.end ();
-		for (j = m_Widgets.begin (); j != jend; j++) {
-			if (*j == m_pWidget)
-				continue;
-			pData = (WidgetData*) g_object_get_data (G_OBJECT (*j), "data");
-			pData->UnselectAll ();
-		}
 		Object *pObject, *Group;
 		set<string> ModifiedObjects;
 		bool modify = false;
@@ -579,12 +487,14 @@ void View::OnCopySelection (GtkWidget* w, GtkClipboard* clipboard)
 
 void View::OnReceive (GtkClipboard* clipboard, GtkSelectionData* selection_data)
 {
-	if ((selection_data->length <= 0) || !selection_data->data)
+	int length = gtk_selection_data_get_length (selection_data);
+	char const *data = reinterpret_cast <char const *> (gtk_selection_data_get_data (selection_data));
+	if ((length <= 0) || !data)
 		return;
 	Application *pApp = m_pDoc->GetApplication ();
 	Tool *pActiveTool = pApp->GetActiveTool ();
 	guint *DataType = (clipboard == gtk_clipboard_get (GDK_SELECTION_CLIPBOARD))? &ClipboardDataType: &ClipboardDataType1;
-	g_return_if_fail ((selection_data->target == gdk_atom_intern (targets[*DataType].target, FALSE)));
+	g_return_if_fail (gtk_selection_data_get_target (selection_data) == gdk_atom_intern (targets[*DataType].target, FALSE));
 	if (pActiveTool->OnReceive (clipboard, selection_data, *DataType))
 		return;
 	else if (pActiveTool) {
@@ -597,13 +507,13 @@ void View::OnReceive (GtkClipboard* clipboard, GtkSelectionData* selection_data)
 	m_pData->UnselectAll ();
 	switch (*DataType) {
 	case GCP_CLIPBOARD_NATIVE:
-		xml = xmlParseMemory ((const char*) selection_data->data, selection_data->length);
+		xml = xmlParseMemory (data, length);
 		m_pDoc->AddData (xml->children->children);
 		xmlFreeDoc (xml);
 		break;
 	case GCP_CLIPBOARD_UTF8_STRING: {
 			Text* text = new Text ();
-			text->SetText ((char const*) selection_data->data);
+			text->SetText (data);
 			text->OnChanged (true);
 			m_pDoc->AddObject (text);
 			m_pData->SetSelected (text);
@@ -611,20 +521,20 @@ void View::OnReceive (GtkClipboard* clipboard, GtkSelectionData* selection_data)
 		break;
 	case GCP_CLIPBOARD_STRING: {
 			Text* text = new Text ();
-			if (!g_utf8_validate ((const char*) selection_data->data, selection_data->length, NULL)) {
+			if (!g_utf8_validate (data, length, NULL)) {
 				gsize r, w;
-				gchar* newstr = g_locale_to_utf8 ((const char*) selection_data->data, selection_data->length, &r, &w, NULL);
+				gchar* newstr = g_locale_to_utf8 ((const char*) data, length, &r, &w, NULL);
 				text->SetText (newstr);
 				g_free (newstr);
 			} else
-				text->SetText ((char const*) selection_data->data);
+				text->SetText (data);
 			text->OnChanged (true);
 			m_pDoc->AddObject (text);
 			m_pData->SetSelected (text);
 		}
 		break;
 	}
-	ArtDRect rect;
+	gccv::Rect rect;
 	double dx, dy;
 	while (gtk_events_pending ())
 		gtk_main_iteration ();
@@ -633,15 +543,16 @@ void View::OnReceive (GtkClipboard* clipboard, GtkSelectionData* selection_data)
 	if (clipboard == gtk_clipboard_get (GDK_SELECTION_CLIPBOARD)) {
 		//center the pasted data at the center of the visible area
 		if (m_bEmbedded) {
-			dx = m_pWidget->allocation.width / 2. - (rect.x0 + rect.x1) / 2.;
-			dy = m_pWidget->allocation.height / 2. - (rect.y0 + rect.y1) / 2.;
+			// might this still occur? what does happen if not everything is visible?
+			dx = m_width / 2. - (rect.x0 + rect.x1) / 2.;
+			dy = m_height / 2. - (rect.y0 + rect.y1) / 2.;
 		} else {
 			GtkAdjustment *horiz, *vert;
 			GtkWidget* parent = gtk_widget_get_parent (m_pWidget);
 			horiz = gtk_viewport_get_hadjustment (GTK_VIEWPORT (parent));
 			vert = gtk_viewport_get_vadjustment (GTK_VIEWPORT (parent));
-			dx = horiz->value + horiz->page_size / 2.  - (rect.x0 + rect.x1) / 2.;
-			dy = vert->value + vert->page_size / 2.  - (rect.y0 + rect.y1) / 2.;
+			dx = gtk_adjustment_get_value (horiz) + gtk_adjustment_get_page_size (horiz) / 2.  - (rect.x0 + rect.x1) / 2.;
+			dy = gtk_adjustment_get_value (vert) + gtk_adjustment_get_page_size (vert) / 2.  - (rect.y0 + rect.y1) / 2.;
 		}
 	} else {
 		//center the pasted data at the mouse click position
@@ -658,7 +569,6 @@ void View::OnReceive (GtkClipboard* clipboard, GtkSelectionData* selection_data)
 	for (i = m_pData->SelectedObjects.begin(); i != end; i++) 
 		pOp->AddObject (*i);
 	m_pDoc->FinishOperation ();
-	gnome_canvas_gcp_update_bounds (GNOME_CANVAS_GCP(m_pData->Canvas));
 }
 
 void View::OnPasteSelection (GtkWidget* w, GtkClipboard* clipboard)
@@ -722,7 +632,7 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 {
 	Application *pApp = m_pDoc->GetApplication ();
 	Tool *pActiveTool = pApp->GetActiveTool ();
-	if (pActiveTool->OnEvent ((GdkEvent*) event))
+	if (pActiveTool->OnKeyPress (event))
 		return true;
 	switch (event->keyval) {
 	case GDK_Delete:
@@ -761,8 +671,7 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 			if ((event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)) != 0 || event->keyval > 127)
 				break;
 			// Now try to get the atom at the cursor
-			Atom *atom = dynamic_cast<Atom*> (m_CurObject);
-			unsigned min_bonds = (atom)? atom->GetTotalBondsNumber (): 0;
+			unsigned min_bonds = (m_CurAtom)? m_CurAtom->GetTotalBondsNumber (): 0;
 			int Z = 0;
 			switch (event->keyval) {
 			case GDK_a:
@@ -845,21 +754,21 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 				break;
 			}
 			if (Z) {
-				if (!atom) {
+				if (!m_CurAtom) {
 					Tools *tools = static_cast<Tools*> (pApp->GetDialog ("tools"));
 					tools->SetElement (Z);
-				} else if (atom->GetZ () != Z && Element::GetElement (Z)->GetMaxBonds () >= min_bonds) {
-					Object *group = atom->GetGroup ();
+				} else if (m_CurAtom->GetZ () != Z && Element::GetElement (Z)->GetMaxBonds () >= min_bonds) {
+					Object *group = m_CurAtom->GetGroup ();
 					Operation *op = m_pDoc->GetNewOperation (GCP_MODIFY_OPERATION);
 					op->AddObject (group);
-					atom->SetZ (Z);
-					Update (atom);
+					m_CurAtom->SetZ (Z);
+					Update (m_CurAtom);
 					// set all bonds as dirty
 					map<gcu::Atom*, gcu::Bond*>::iterator i;
-					Bond *bond = reinterpret_cast <Bond *> (atom->GetFirstBond (i));
+					Bond *bond = reinterpret_cast <Bond *> (m_CurAtom->GetFirstBond (i));
 					while (bond) {
 						bond->SetDirty ();
-						bond = reinterpret_cast <Bond *> (atom->GetNextBond (i));
+						bond = reinterpret_cast <Bond *> (m_CurAtom->GetNextBond (i));
 					}
 					op->AddObject (group, 1);
 					m_pDoc->FinishOperation ();
@@ -891,7 +800,7 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 				symbol = (*i).first;
 				symbol.insert (((symbol.length () > 1)? 1: 0), "_");
 				action = GTK_ACTION (gtk_action_new ((*i).second->GetSymbol (), symbol.c_str (), (*i).second->GetName (), NULL));
-				g_signal_connect (action, "activate", G_CALLBACK (do_set_symbol), (atom)? static_cast<Object*> (atom): static_cast<Object*> (m_pDoc));
+				g_signal_connect (action, "activate", G_CALLBACK (do_set_symbol), (m_CurAtom)? static_cast<Object*> (m_CurAtom): static_cast<Object*> (m_pDoc));
 				gtk_action_group_add_action (group, action);
 				g_object_unref (action);
 				ui = string ("<ui><popup><menuitem action='") + (*i).second->GetSymbol () + "'/></popup></ui>";
@@ -907,7 +816,7 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 	return false;
 }
 
-bool View::OnKeyRelease (GtkWidget* w, GdkEventKey* event)
+bool View::OnKeyRelease (G_GNUC_UNUSED GtkWidget* w, GdkEventKey* event)
 {
 	Application *pApp = m_pDoc->GetApplication ();
 	Tool *pActiveTool = pApp->GetActiveTool ();
@@ -940,37 +849,8 @@ bool View::OnKeyRelease (GtkWidget* w, GdkEventKey* event)
 	}
 	return false;
 }
-	
-bool View::OnSize (GtkWidget *w, int width, int height)
-{
-	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
-	gnome_canvas_set_scroll_region (GNOME_CANVAS (w), 0, 0, (double) width / pData->Zoom, (double) height / pData->Zoom);
-	if (pData->Background)
-		g_object_set (G_OBJECT (pData->Background), "x2", (double) width / pData->Zoom, "y2", (double) height / pData->Zoom, NULL);
-	return true;
-}
 
-void View::UpdateSize (double x1, double y1, double x2, double y2)
-{
-	if (x1 < 0.0) x2 -= x1;
-	if (y1 < 0.0) y2 -= y1;
-	list<GtkWidget*>::iterator i, end = m_Widgets.end ();
-	if ((x2 != m_width) || (y2 != m_height))
-		for (i = m_Widgets.begin(); i != end; i++) {
-			WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (*i), "data");
-			gtk_widget_set_size_request (*i, (int) ceil (x2 * pData->Zoom), (int) ceil (y2 * pData->Zoom));
-		}
-	if ((x1 < 0.0) || (y1 < 0.0))
-	{
-		x1 = - x1;
-		y1 = - y1;
-		Theme* pTheme = m_pDoc->GetTheme ();
-		m_pDoc->Move(x1 / pTheme->GetZoomFactor (), y1 / pTheme->GetZoomFactor ());
-		Update (m_pDoc);
-	}
-}
-
-void View::SetGnomeCanvasPangoActive (GnomeCanvasPango* item)
+void View::SetTextActive (gccv::Text* item)
 {
 	m_ActiveRichText = item;
 	m_Dragging = false;
@@ -1017,7 +897,7 @@ static cairo_status_t cairo_write_func (void *closure, const unsigned char *data
 
 void View::ExportImage (string const &filename, const char* type, int resolution)
 {
-	ArtDRect rect;
+	gccv::Rect rect;
 	m_pData->GetObjectBounds (m_pDoc, &rect);
 	m_pData->ShowSelection (false);
 	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
@@ -1032,7 +912,7 @@ void View::ExportImage (string const &filename, const char* type, int resolution
 			gtk_widget_destroy (message);
 			g_error_free (error);
 		}
-		ArtDRect rect;
+		gccv::Rect rect;
 		m_pData->GetObjectBounds (m_pDoc, &rect);
 		cairo_surface_t *surface = NULL;
 		if (!strcmp (type, "pdf"))
@@ -1048,12 +928,27 @@ void View::ExportImage (string const &filename, const char* type, int resolution
 		cairo_surface_destroy (surface);
 		Render (cr);
 		cairo_destroy (cr);
+		g_object_unref (output);
 	} else if (!strcmp (type, "svg")) {
-		xmlDocPtr doc = BuildSVG ();
-		xmlIndentTreeOutput = true;
-		xmlKeepBlanksDefault (0);
-		xmlSaveFormatFile (filename.c_str (), doc, true);
-		xmlFreeDoc (doc);
+		GError *error = NULL;
+		GsfOutput *output = gsf_output_gio_new_for_uri (filename.c_str (), &error);
+		if (error) {
+			gchar * mess = g_strdup_printf (_("Could not create stream!\n%s"), error->message);
+			GtkWidget* message = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (m_pWidget)), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess);
+			g_free (mess);
+			gtk_dialog_run (GTK_DIALOG (message));
+			gtk_widget_destroy (message);
+			g_error_free (error);
+		}
+		gccv::Rect rect;
+		m_pData->GetObjectBounds (m_pDoc, &rect);
+		cairo_surface_t *surface = cairo_svg_surface_create_for_stream (cairo_write_func, output, w, h);
+		cairo_t *cr = cairo_create (surface);
+		cairo_translate (cr, -rect.x0, -rect.y0);
+		cairo_surface_destroy (surface);
+		Render (cr);
+		cairo_destroy (cr);
+		g_object_unref (output);
 	} else {
 		GdkPixbuf *pixbuf = BuildPixbuf (resolution);
 		GFile *file = g_vfs_get_file_for_uri (g_vfs_get_default (), filename.c_str ());
@@ -1071,59 +966,51 @@ void View::ExportImage (string const &filename, const char* type, int resolution
 	m_pData->ShowSelection (true);
 }
 
-xmlDocPtr View::BuildSVG ()
+char *View::BuildSVG ()
 {
-	ArtDRect rect;
+	gccv::Rect rect;
 	m_pData->GetObjectBounds (m_pDoc, &rect);
-	xmlDocPtr doc = xmlNewDoc ((const xmlChar*)"1.0");
-	char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	char *buf;
-	setlocale (LC_NUMERIC, "C");
-	xmlNewDtd (doc,
-		   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
-		   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
-	xmlDocSetRootElement (doc, xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL));
-	xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
-	xmlSetNs (doc->children, ns);
-	xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
-	rect.x0 = floor (rect.x0);
-	rect.y0 = floor (rect.y0);
-	rect.x1 = ceil (rect.x1);
-	rect.y1 = ceil (rect.y1);
-	buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
-	xmlNewProp (doc->children, (const xmlChar*)"width", (const xmlChar*)buf);
-	g_free (buf);
-	buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
-	xmlNewProp (doc->children, (const xmlChar*)"height", (const xmlChar*)buf);
-	g_free (buf);
-	xmlNodePtr node;
-	node = xmlNewDocNode (doc, NULL, (const xmlChar*)"rect", NULL);
-	xmlAddChild (doc->children, node);
-	buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
-	xmlNewProp (node, (const xmlChar*)"width", (const xmlChar*)buf);
-	g_free (buf);
-	buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
-	xmlNewProp (node, (const xmlChar*)"height", (const xmlChar*)buf);
-	g_free (buf);
-	xmlNewProp (node, (const xmlChar*)"stroke", (const xmlChar*)"none");
-	xmlNewProp (node, (const xmlChar*)"fill", (const xmlChar*)"white");
-	if (rect.x0 != 0. || rect.y0 != 0) {
-		node = xmlNewDocNode (doc, NULL, (const xmlChar*)"g", NULL);
-		xmlAddChild (doc->children, node);
-		buf = g_strdup_printf ("translate(%g,%g)", - rect.x0, - rect.y0);
-		xmlNewProp (node, (const xmlChar*)"transform", (const xmlChar*)buf);
-		g_free (buf);
-	} else
-		node = doc->children;
-	g_printable_export_svg (G_PRINTABLE (m_pData->Group), doc, node);
-	setlocale (LC_NUMERIC, old_num_locale);
-	g_free (old_num_locale);
-	return doc;
+	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
+	GsfOutput *output = gsf_output_memory_new ();
+	cairo_surface_t *surface = cairo_svg_surface_create_for_stream (cairo_write_func, output, w, h);
+	cairo_t *cr = cairo_create (surface);
+	cairo_translate (cr, -rect.x0, -rect.y0);
+	cairo_surface_destroy (surface);
+	Render (cr);
+	cairo_destroy (cr);
+	m_pData->ShowSelection (true);
+	char *m = g_strdup (reinterpret_cast <char const*> (gsf_output_memory_get_bytes (reinterpret_cast <GsfOutputMemory *> (output))));
+	g_object_unref (output);
+	return m;
+}
+
+char *View::BuildEPS ()
+{
+	gccv::Rect rect;
+	m_pData->GetObjectBounds (m_pDoc, &rect);
+	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
+	GsfOutput *output = gsf_output_memory_new ();
+	cairo_surface_t *surface = cairo_ps_surface_create_for_stream (cairo_write_func, output, w, h);
+	cairo_ps_surface_set_eps (surface, TRUE);
+	cairo_t *cr = cairo_create (surface);
+	cairo_translate (cr, -rect.x0, -rect.y0);
+	cairo_surface_destroy (surface);
+	Render (cr);
+	cairo_destroy (cr);
+	m_pData->ShowSelection (true);
+	char *m = g_strdup (reinterpret_cast <char const*> (gsf_output_memory_get_bytes (reinterpret_cast <GsfOutputMemory *> (output))));
+	g_object_unref (output);
+	return m;
+}
+
+static void destroy_surface (G_GNUC_UNUSED guchar *pixels, gpointer data)
+{
+	cairo_surface_destroy (reinterpret_cast <cairo_surface_t *> (data));
 }
 
 GdkPixbuf *View::BuildPixbuf (int resolution)
 {
-	ArtDRect rect;
+	gccv::Rect rect;
 	m_pData->GetObjectBounds (m_pDoc, &rect);
 	m_pData->ShowSelection (false);
 	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
@@ -1135,46 +1022,57 @@ GdkPixbuf *View::BuildPixbuf (int resolution)
 		h = (int) rint ((double) h * zoom);
 	} else
 		zoom = 1.;
-	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
-	gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
-	GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
-	gdk_pixbuf_fill (pixbuf, 0xffffffff);
-	GnomeCanvasBuf buf;
-	buf.buf = gdk_pixbuf_get_pixels (pixbuf);
-	buf.rect.x0 = (int) floor (rect.x0 * zoom);
-	buf.rect.x1 = (int) ceil (rect.x1 * zoom);
-	buf.rect.y0 = (int) floor (rect.y0 * zoom);
-	buf.rect.y1 = (int) ceil (rect.y1 * zoom);
-	buf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-	buf.bg_color = 0xffffff;
-	buf.is_buf = 1;
-	(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &buf);
-	// restore zoom level
-	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), m_pData->Zoom);
+	cairo_surface_t *surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
+	cairo_t *cr = cairo_create (surface);
+	if (m_pDoc->GetApp () && !m_pDoc->GetApp ()->GetTransparentBackground ()) {
+		cairo_set_source_rgb (cr, 1., 1., 1.);
+		cairo_paint (cr);
+	}
+	cairo_translate (cr, -floor (rect.x0), -floor (rect.y0));
+	cairo_scale (cr, zoom, zoom);
+	m_Canvas->Render (cr, false);
+	int rowstride = cairo_image_surface_get_stride (surface);
+	unsigned char *data = cairo_image_surface_get_data (surface);
+	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_data (data, GDK_COLORSPACE_RGB, TRUE, 8, w, h, rowstride, destroy_surface, surface);
+	go_cairo_convert_data_to_pixbuf (data, NULL, w, h, rowstride);
+	cairo_destroy (cr);
 	return pixbuf;
 }
 
 void View::EnsureSize ()
 {
-	GnomeCanvas *canvas = GNOME_CANVAS (m_pWidget);
-	gnome_canvas_update_now (canvas);
-	if (GTK_WIDGET_REALIZED (m_pWidget))
-		g_signal_emit_by_name (m_pWidget, "update_bounds");
+	double x1, y1, x2, y2;
+	m_Canvas->GetRoot ()->GetBounds (x1, y1, x2, y2);
+	if (x1 < 0.) x2 -= x1;
+	if (y1 < 0.) y2 -= y1;
+	if (x2 <= 0. || y2 <= 0.)
+		return;
+	if ((x2 != m_width) || (y2 != m_height)) {
+		m_width = x2;
+		m_height = y2;
+		gtk_widget_set_size_request (m_pWidget, (int) ceil (x2 * m_Canvas->GetZoom ()), (int) ceil (y2 * m_Canvas->GetZoom ()));
+	}
+	if ((x1 < 0.) || (y1 < 0.))
+	{
+		Theme* pTheme = m_pDoc->GetTheme ();
+		x1 = (x1 < 0.)? -x1 / pTheme->GetZoomFactor (): 0.;
+		y1 = (y1 < 0.)? -y1 / pTheme->GetZoomFactor (): 0.;
+		m_pDoc->Move(x1, y1);
+		Update (m_pDoc);
+	}
 }
 
 void View::Zoom (double zoom)
 {
 	m_pData->Zoom = zoom;
-	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
+	m_Canvas->SetZoom (zoom);
 	EnsureSize ();
-	// Call OnSize to be certain that the canvas scroll region will be correct
-	OnSize (m_pWidget, m_width, m_height);
 }
 
 void View::ShowCursor (bool show)
 {
 	if (m_ActiveRichText)
-		g_object_set (G_OBJECT (m_ActiveRichText), "editing", show, NULL);
+		m_ActiveRichText->SetEditing (show);
 }
 
 void View::UpdateTheme ()
@@ -1208,17 +1106,124 @@ void View::UpdateTheme ()
 void View::Render (cairo_t *cr)
 {
 	m_pData->ShowSelection(false);
-	Object* pObj = NULL;
-	if (m_ActiveRichText) {
+//	Object* pObj = NULL;
+/*	if (m_ActiveRichText) {
 		pObj = (Object*) g_object_get_data (G_OBJECT (m_ActiveRichText), "object");
 		if (pObj) pObj->SetSelected (m_pWidget, SelStateUnselected);
-	}
-	GnomeCanvas *canvas = GNOME_CANVAS (m_pWidget);
+	}*/
+	m_Canvas->Render (cr, true);
+/*	GnomeCanvas *canvas = GNOME_CANVAS (m_pWidget);
 	gnome_canvas_update_now (canvas);
-	g_printable_draw_cairo (G_PRINTABLE (m_pData->Group), cr);
+	g_printable_draw_cairo (G_PRINTABLE (m_pData->Group), cr);*/
 	m_pData->ShowSelection (true);
-	if (pObj)
-		pObj->SetSelected (m_pWidget, SelStateUpdating);
+/*	if (pObj)
+		pObj->SetSelected (SelStateUpdating);*/
+}
+
+// Events
+
+bool View::OnButtonPressed (gccv::ItemClient *client, unsigned button, double x, double y, unsigned state)
+{
+	Application *App = m_pDoc->GetApplication ();
+	Theme *pTheme = m_pDoc->GetTheme ();
+	Tool* pActiveTool = App? App->GetActiveTool (): NULL;
+	Object *pAtom;
+	m_CurObject = dynamic_cast <Object *> (client);
+	if (m_CurObject && ((pAtom = m_CurObject->GetAtomAt (x / pTheme->GetZoomFactor (), y / pTheme->GetZoomFactor ()))))
+			m_CurObject = pAtom;
+	if ((!m_pDoc->GetEditable ()) || (!pActiveTool))
+		return true;
+	switch (button) {
+	case 1: {
+		if (m_Dragging) break;
+		bool result = pActiveTool->OnClicked (this, m_CurObject, x, y, state);
+/*		if (item && (item == (GnomeCanvasItem*)m_ActiveRichText)) {
+			GnomeCanvasItemClass* klass = GNOME_CANVAS_ITEM_CLASS (((GTypeInstance*) item)->g_class);
+			return klass->event (item, event);
+		}*/
+		m_Dragging = result;
+		return true;
+	}
+	case 2: {
+		m_lastx = x;
+		m_lasty = y;
+		GtkClipboard *clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
+		OnPasteSelection (m_pWidget, clipboard);
+		return true;
+	}
+	case 3: {
+		bool result;
+		g_object_unref (m_UIManager);
+		m_UIManager = gtk_ui_manager_new ();
+		result = pActiveTool->OnRightButtonClicked (this, m_CurObject, x, y, m_UIManager);
+		if (m_CurObject)
+			result |= m_CurObject->BuildContextualMenu (m_UIManager, m_CurObject, x / GetZoomFactor (), y / GetZoomFactor ());
+		if (result) {
+			GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager, "/popup");
+			gtk_menu_popup (GTK_MENU (w), NULL, NULL, NULL, NULL, 3,  gtk_get_current_event_time ());
+			return true;
+		}
+	}
+	}
+	return true;
+}
+
+bool View::OnButtonReleased (G_GNUC_UNUSED gccv::ItemClient *client, unsigned button, double x, double y, unsigned state)
+{
+	Application *App = m_pDoc->GetApplication ();
+	Tool* pActiveTool = App? App->GetActiveTool (): NULL;
+	if ((!m_pDoc->GetEditable ()) || (!pActiveTool))
+		return true;
+	switch (button) {
+	case 1:
+		if (!m_Dragging)
+			break;
+		m_Dragging = false;
+		pActiveTool->OnRelease (x, y, state);
+		m_pDoc->GetApplication ()->ClearStatus ();
+		return true;
+	}
+	return true;
+}
+
+bool View::OnDrag (G_GNUC_UNUSED gccv::ItemClient *client, double x, double y, unsigned state)
+{
+	Application *App = m_pDoc->GetApplication ();
+	Tool* pActiveTool = App? App->GetActiveTool (): NULL;	
+	if (m_pDoc->GetEditable () && pActiveTool && m_Dragging)
+		pActiveTool->OnDrag (x, y, state);
+	return true;
+}
+
+bool View::OnMotion (gccv::ItemClient *client, double x, double y, unsigned state)
+{
+	m_CurObject = dynamic_cast <Object *> (client);
+	if (m_CurObject) {
+		m_CurAtom = dynamic_cast <Atom *> (m_CurObject);
+		if (!m_CurAtom) {
+			Theme *theme = m_pDoc->GetTheme ();
+			m_CurAtom = reinterpret_cast <Atom *> (m_CurObject->GetAtomAt (x / theme->GetZoomFactor (), y / theme->GetZoomFactor ()));
+		}
+	} else
+		m_CurAtom = NULL;
+	Application *App = m_pDoc->GetApplication ();
+	Tool* pActiveTool = App? App->GetActiveTool (): NULL;	
+	if (m_pDoc->GetEditable () && pActiveTool)
+		pActiveTool->OnMotion (this, ((m_CurAtom)? m_CurAtom: m_CurObject), x, y, state);
+	return true;
+}
+
+void View::SetSelectionState (Object *object, int state)
+{
+	gccv::ItemClient *client = dynamic_cast <gccv::ItemClient *> (object);
+	if (client)
+		client->SetSelected (state);
+	map<string, Object*>::iterator i;
+	Object *child = object->GetFirstChild (i);
+	while (child) {
+		SetSelectionState (child, state);
+		child = object->GetNextChild (i);
+	}
 }
 
 }	//	namespace gcp
