@@ -31,7 +31,10 @@
 #include "operation.h"
 #include "theme.h"
 #include "tool.h"
-#include <goffice/math/go-math.h>
+#include <gccv/canvas.h>
+#include <gccv/group.h>
+#include <gccv/item-client.h>
+#include <gccv/item.h>
 #include <cstring>
 
 using namespace gcu;
@@ -60,7 +63,7 @@ void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_da
 {
 	GtkClipboard* sel_clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 	guint *DataType = (clipboard == sel_clipboard)? &ClipboardDataType: &ClipboardDataType1;
-	if (selection_data->target == gdk_atom_intern ("TARGETS", FALSE)) {
+	if (gtk_selection_data_get_target (selection_data) == gdk_atom_intern ("TARGETS", FALSE)) {
 		static char const *formats [] =
 		{
 			GCHEMPAINT_ATOM_NAME,
@@ -75,11 +78,12 @@ void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_da
 			NULL
 		};
 
-		GdkAtom const *targets = (GdkAtom *) selection_data->data;
-		unsigned const atom_count = (selection_data->length / sizeof (GdkAtom));
+		GdkAtom const *targets = (GdkAtom *) gtk_selection_data_get_data (selection_data);
+		int length = gtk_selection_data_get_length (selection_data);
+		unsigned const atom_count = (length / sizeof (GdkAtom));
 		unsigned i, j;
 		/* Nothing on clipboard? */
-		if (selection_data->length < 0) {
+		if (length < 0) {
 			if (clipboard == sel_clipboard)
 				App->ActivateWindowsActionWidget ("/MainMenu/EditMenu/Paste", false);
 			return;
@@ -125,14 +129,19 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		View *pView = Doc->GetView ();
 		pView->CreateNewWidget (); // force canvas creation
 		Doc->ParseXMLTree (pDoc);
-		xmlDocPtr doc = pView->BuildSVG ();
-		xmlDocDumpFormatMemory (doc, &ClipboardData, &size, info);
-		gtk_selection_data_set (selection_data, gdk_atom_intern (export_targets[info].target, FALSE), 8, (const guchar*) ClipboardData, size);
-		xmlFreeDoc (doc);
+		ClipboardTextData = pView->BuildSVG ();
+		gtk_selection_data_set_text (selection_data, ClipboardTextData, strlen (ClipboardTextData));
 		delete Doc;
 		break;
 	}
 	case GCP_CLIPBOARD_EPS: {
+		Document *Doc = new Document (NULL, true);
+		View *pView = Doc->GetView ();
+		pView->CreateNewWidget (); // force canvas creation
+		Doc->ParseXMLTree (pDoc);
+		ClipboardTextData = pView->BuildEPS ();
+		gtk_selection_data_set_text (selection_data, ClipboardTextData, strlen (ClipboardTextData));
+		delete Doc;
 		break;
 	}
 	case GCP_CLIPBOARD_PNG: {
@@ -217,8 +226,7 @@ bool WidgetData::IsSelected (Object const *obj) const
 void WidgetData::Unselect (Object *obj)
 {
 	SelectedObjects.remove (obj);
-	obj->SetSelected (Canvas, SelStateUnselected);
-	m_View->Update (obj);
+	m_View->SetSelectionState (obj, SelStateUnselected);
 }
 
 void WidgetData::UnselectAll ()
@@ -227,8 +235,7 @@ void WidgetData::UnselectAll ()
 	while (!SelectedObjects.empty ()) {
 		obj = SelectedObjects.front ();
 		SelectedObjects.pop_front ();
-		obj->SetSelected (Canvas, SelStateUnselected);
-		m_View->Update (obj);	//FIXME: is it really useful
+		Unselect (obj);
 	}
 }
 
@@ -236,7 +243,7 @@ void WidgetData::SetSelected (Object *obj)
 {
 	if (!IsSelected (obj)) {
 		SelectedObjects.push_front (obj);
-		obj->SetSelected (Canvas, SelStateSelected);
+		m_View->SetSelectionState (obj, SelStateSelected);
 	}
 }
 
@@ -250,11 +257,10 @@ void WidgetData::MoveSelectedItems (double dx, double dy)
 void WidgetData::MoveItems (Object* obj, double dx, double dy)
 {
 	Object* pObject;
-	GnomeCanvasGroup* group;
-	if ((group = Items[obj]))
-		gnome_canvas_item_move ((GnomeCanvasItem*) group, dx, dy);
-	else
-		Items.erase (obj);
+	gccv::ItemClient *client = dynamic_cast <gccv::ItemClient *> (obj);
+	gccv::Item *item = (client)? client->GetItem (): NULL;
+	if (item)
+		item->Move (dx, dy);
 	std::map<std::string, Object*>::iterator i;
 	pObject = obj->GetFirstChild (i);
 	while (pObject) {
@@ -316,15 +322,14 @@ void WidgetData::Copy (GtkClipboard* clipboard)
 	gtk_clipboard_request_contents (clipboard, gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, App);
 }
 
-void WidgetData::GetObjectBounds (Object const *obj, ArtDRect &rect) const
+void WidgetData::GetObjectBounds (Object const *obj, gccv::Rect &rect) const
 {
 	Object const *pObject;
-	GnomeCanvasGroup const *group;
-	std::map<gcu::Object const *, GnomeCanvasGroup*>::const_iterator g;
+	gccv::ItemClient const *client;
 	double x1, y1, x2,  y2;
-	if ((g = Items.find (obj)) != Items.end ()) {
-		group = (*g).second;
-		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (group), &x1, &y1, &x2, &y2);
+	client = dynamic_cast <gccv::ItemClient const *> (obj);
+	if (client && client->GetItem ()) {
+		client->GetItem ()->GetBounds (x1, y1, x2, y2);
 		if (x2 > 0.) {
 			if (!go_finite (rect.x0)) {
 				rect.x0 = x1;
@@ -347,7 +352,7 @@ void WidgetData::GetObjectBounds (Object const *obj, ArtDRect &rect) const
 	}
 }
 
-void WidgetData::GetSelectionBounds (ArtDRect &rect) const
+void WidgetData::GetSelectionBounds (gccv::Rect &rect) const
 {
 	std::list<Object*>::const_iterator i, end = SelectedObjects.end ();
 	rect.x0 = go_nan;
@@ -357,7 +362,7 @@ void WidgetData::GetSelectionBounds (ArtDRect &rect) const
 		rect.x0 = rect.y0 = rect.x1 = rect.y1 = 0.;
 }
 
-void WidgetData::GetObjectBounds (Object const *obj, ArtDRect *rect) const
+void WidgetData::GetObjectBounds (Object const *obj, gccv::Rect *rect) const
 {
 	rect->x0 = go_nan;
 	GetObjectBounds (obj, *rect);
@@ -367,16 +372,21 @@ void WidgetData::GetObjectBounds (Object const *obj, ArtDRect *rect) const
 
 void WidgetData::SelectAll ()
 {
-	std::map<Object const*, GnomeCanvasGroup*>::iterator i, end = Items.end ();
-	Object *pObject;
-	for (i = Items.begin (); i != end; i++) {
-		pObject = (*i).first->GetGroup ();
-		if (pObject) {
-			if (!IsSelected (pObject))
-				SetSelected (pObject);
-		}
-		else if (!IsSelected ((*i).first))
-			SetSelected (const_cast <Object *> ((*i).first));
+	std::list <gccv::Item *>::iterator it;
+	gccv::Group *root = m_View->GetCanvas ()->GetRoot ();
+	gccv::Item *item;
+	Object *pObject, *pGroup;
+	for (item = root->GetFirstChild (it); item; item = root->GetNextChild (it)) {
+		if (item->GetClient ())
+			pObject = dynamic_cast <Object *> (item->GetClient ());
+		else
+			continue;
+		pGroup = pObject->GetGroup ();
+		if (pGroup) {
+			if (!IsSelected (pGroup))
+				SetSelected (pGroup);
+		} else if (!IsSelected (pObject))
+			SetSelected (pObject);
 	}
 }
 
@@ -389,7 +399,7 @@ void WidgetData::ShowSelection (bool state)
 {
 	std::list<Object*>::iterator i, end = SelectedObjects.end ();
 	for (i = SelectedObjects.begin (); i != end; i++)
-		(*i)->SetSelected (Canvas, (state)? SelStateSelected: SelStateUnselected);
+		m_View->SetSelectionState (*i, (state)? SelStateSelected: SelStateUnselected);
 }
 
 }	//	namespace gcp
