@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/glview.cc 
  *
- * Copyright (C) 2006-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2006-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -27,11 +27,6 @@
 #include <gcu/gldocument.h>
 #include <gcu/glview.h>
 #include <gcu/macros.h>
-#ifndef GOFFICE_HAS_GLOBAL_HEADER
-#   include <goffice/goffice-features.h>
-#   include <goffice/utils/go-image.h>
-#   include <gio/gio.h>
-#endif
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <gtk/gtkgl.h>
@@ -51,28 +46,24 @@ using namespace std;
 
 namespace gcu
 {
-#ifdef HAVE_GO_CONF_SYNC
 GOConfNode *GLView::m_ConfNode = NULL;
-#else
-GConfClient *GLView::m_ConfClient = NULL;
-#endif
 guint GLView::m_NotificationId = 0;
 int GLView::nbViews = 0;
 
 // Callbacks
-static bool on_init(GtkWidget *widget, GLView* View) 
+static bool on_init(G_GNUC_UNUSED GtkWidget *widget, GLView* View) 
 {
 	View->Init ();
 	return true;
 }
 
-bool on_reshape(GtkWidget *widget, GdkEventConfigure *event, GLView* View) 
+bool on_reshape(G_GNUC_UNUSED GtkWidget *widget, GdkEventConfigure *event, GLView* View) 
 {
-	View->Reshape ();
+	View->Reshape (event->width, event->height);
 	return true;
 }
 
-static bool on_draw(GtkWidget *widget, GdkEventExpose *event, GLView* View) 
+static bool on_draw(G_GNUC_UNUSED GtkWidget *widget, GdkEventExpose *event, GLView* View) 
 {
 	/* Draw only last expose. */
 	if (event->count > 0) return TRUE;
@@ -81,30 +72,22 @@ static bool on_draw(GtkWidget *widget, GdkEventExpose *event, GLView* View)
 	return true;
 }
 
-static bool on_motion(GtkWidget *widget, GdkEventMotion *event, GLView* View) 
+static bool on_motion(G_GNUC_UNUSED GtkWidget *widget, GdkEventMotion *event, GLView* View) 
 {
 	View->OnMotion (event);
 	return true;
 }
 
-static bool on_pressed(GtkWidget *widget, GdkEventButton *event, GLView* View) 
+static bool on_pressed(G_GNUC_UNUSED GtkWidget *widget, GdkEventButton *event, GLView* View) 
 {
 	return View->OnPressed (event);
 }
 
-#ifdef HAVE_GO_CONF_SYNC
-static void on_config_changed (GOConfNode *node, gchar const *key, gpointer data)
+static void on_config_changed (GOConfNode *node, gchar const *key, G_GNUC_UNUSED gpointer data)
 {
 	if (!strcmp (key, ROOTDIR"off-screen-rendering"))
 		OffScreenRendering = go_conf_get_bool (node, key);
 }
-#else
-static void on_config_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer data)
-{
-	if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"off-screen-rendering"))
-		OffScreenRendering = gconf_value_get_bool (gconf_entry_get_value (entry));
-}
-#endif
 
 // GLView implementation
 #define GCU_CONF_DIR_GL "gl"
@@ -133,19 +116,9 @@ GLView::GLView (GLDocument* pDoc) throw (std::runtime_error): Printable ()
 											GDK_GL_MODE_DOUBLE));
 		if (glconfig == NULL)
 			throw  runtime_error ("*** Cannot find the double-buffered visual.\n");
-#ifdef HAVE_GO_CONF_SYNC
 		m_ConfNode = go_conf_get_node (Application::GetConfDir (), GCU_CONF_DIR_GL);
-#else
-		GError *error = NULL;
-		m_ConfClient = gconf_client_get_default ();
-		gconf_client_add_dir (m_ConfClient, "/apps/gchemutils/gl", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-#endif
 		GCU_GCONF_GET_NO_CHECK ("off-screen-rendering", bool, OffScreenRendering, true)
-#ifdef HAVE_GO_CONF_SYNC
 		m_NotificationId = go_conf_add_monitor (m_ConfNode, "off-screen-rendering", (GOConfMonitorFunc) on_config_changed, NULL);
-#else
-		m_NotificationId = gconf_client_notify_add (m_ConfClient, "/apps/gchemutils/gl", (GConfClientNotifyFunc) on_config_changed, NULL, NULL, NULL);
-#endif
 	}
 	/* create new OpenGL widget */
 	m_pWidget = GTK_WIDGET(gtk_drawing_area_new());
@@ -158,12 +131,11 @@ GLView::GLView (GLDocument* pDoc) throw (std::runtime_error): Printable ()
 					GDK_GL_RGBA_TYPE);
 	
 	gtk_widget_set_events(GTK_WIDGET(m_pWidget),
-		GDK_EXPOSURE_MASK |
-		GDK_POINTER_MOTION_MASK |
-		GDK_POINTER_MOTION_HINT_MASK |
-		GDK_BUTTON_PRESS_MASK |
-	    GDK_BUTTON_RELEASE_MASK);
-
+	GDK_EXPOSURE_MASK|
+	GDK_BUTTON_MOTION_MASK|
+	GDK_POINTER_MOTION_HINT_MASK|
+	GDK_BUTTON_PRESS_MASK);
+	
 	// Connect signal handlers
 	// Do initialization when widget has been realized.
 	g_signal_connect (G_OBJECT (m_pWidget), "realize",
@@ -188,16 +160,9 @@ GLView::~GLView ()
 {
 	nbViews--;
 	if (!nbViews) {
-#ifdef HAVE_GO_CONF_SYNC
 		go_conf_remove_monitor (m_NotificationId);
 		go_conf_free_node (m_ConfNode);
 		m_ConfNode = NULL;
-#else
-		gconf_client_notify_remove (m_ConfClient, m_NotificationId);
-		gconf_client_remove_dir (m_ConfClient, "/apps/gchemutils/gl", NULL);
-		g_object_unref (m_ConfClient);
-		m_ConfClient = NULL;
-#endif
 		m_NotificationId = 0;
 	}
 }
@@ -225,16 +190,18 @@ void GLView::Init ()
     }
 }
 
-void GLView::Reshape ()
+void GLView::Reshape (int width, int height)
 {
+	m_WindowWidth = width;
+	m_WindowHeight = height;
 	if (!m_bInit)
 		return;
 	float fAspect;
 	GdkGLContext *glcontext = gtk_widget_get_gl_context (m_pWidget);
 	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (m_pWidget);
 	if (gdk_gl_drawable_gl_begin (gldrawable, glcontext)) {
-		if (m_pWidget->allocation.height) {
-			fAspect = (GLfloat) m_pWidget->allocation.width / (GLfloat) m_pWidget->allocation.height;
+		if (height) {
+			fAspect = (GLfloat) width / (GLfloat) height;
 			if (fAspect == 0.0)
 				fAspect = 1.0;
 		} else	// don't divide by zero, not that we should ever run into that...
@@ -242,7 +209,7 @@ void GLView::Reshape ()
 		double x = m_Doc->GetMaxDist ();
 		if (x == 0)
 			x = 1;
-		glViewport (0,0, m_pWidget->allocation.width, m_pWidget->allocation.height);
+		glViewport (0,0, width, height);
 		if (fAspect > 1.0) {
 			m_Height = x * (1 - tan (m_Angle / 360 * M_PI));
 			m_Width = m_Height * fAspect;
@@ -296,7 +263,7 @@ void GLView::Update()
 		m_Doc->Draw (m_Euler);
 		gdk_gl_drawable_gl_end (gldrawable);
     }
-	Reshape ();
+	Reshape (m_WindowWidth, m_WindowHeight);
 	Draw ();
 }
 
@@ -339,7 +306,7 @@ bool GLView::OnMotion(GdkEventMotion *event)
 		Rotate (x - m_Lastx, y - m_Lasty);
 		m_Lastx = x;
 		m_Lasty = y;
-		gtk_widget_queue_draw_area(m_pWidget, 0, 0, m_pWidget->allocation.width, m_pWidget->allocation.height);
+		gtk_widget_queue_draw_area(m_pWidget, 0, 0, m_WindowWidth, m_WindowHeight);
 	}
 	return true;
 }
@@ -478,14 +445,14 @@ GdkPixbuf *GLView::BuildPixbuf (unsigned width, unsigned height) const
 		gtk_window_present (GTK_WINDOW (gtk_widget_get_toplevel (m_pWidget))); 
 		while (gtk_events_pending ())
 			gtk_main_iteration ();
-		if (m_pWidget->allocation.width & (s - 1))
-			LineWidth = ((~(s - 1)) & (m_pWidget->allocation.width * 3)) + s;
+		if (m_WindowWidth & (s - 1))
+			LineWidth = ((~(s - 1)) & (m_WindowWidth * 3)) + s;
 		else
-			LineWidth = m_pWidget->allocation.width * 3;
-		unsigned size = LineWidth * m_pWidget->allocation.height;
+			LineWidth = m_WindowWidth * 3;
+		unsigned size = LineWidth * m_WindowHeight;
 		int i, j;
-		hstep = m_pWidget->allocation.width;
-		vstep = m_pWidget->allocation.height;
+		hstep = m_WindowWidth;
+		vstep = m_WindowHeight;
 		tmp = new unsigned char[size];
 		if (!tmp)
 			goto osmesa;
@@ -523,7 +490,7 @@ GdkPixbuf *GLView::BuildPixbuf (unsigned width, unsigned height) const
 					gdk_gl_drawable_gl_end (gldrawable);
 					glPixelStorei (GL_PACK_ALIGNMENT, s);
 					glReadBuffer (GL_BACK_LEFT);
-					glReadPixels (0, 0, m_pWidget->allocation.width, m_pWidget->allocation.height, GL_RGB,
+					glReadPixels (0, 0, m_WindowWidth, m_WindowHeight, GL_RGB,
 										GL_UNSIGNED_BYTE, tmp);
 					// copy the data to the pixbuf.
 					// linesize
@@ -557,7 +524,7 @@ osmesa:
 	return pixbuf;
 }
 
-void GLView::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
+void GLView::DoPrint (G_GNUC_UNUSED GtkPrintOperation *print, GtkPrintContext *context, G_GNUC_UNUSED int page) const
 {
 	cairo_t *cr;
 	gdouble width, height;
@@ -566,8 +533,8 @@ void GLView::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
 	width = gtk_print_context_get_width (context);
 	height = gtk_print_context_get_height (context);
 	int w, h; // size in points
-	w = m_pWidget->allocation.width;
-	h = m_pWidget->allocation.height;
+	w = m_WindowWidth;
+	h = m_WindowHeight;
 	switch (GetScaleType ()) {
 	case GCU_PRINT_SCALE_NONE:
 		break;
