@@ -25,15 +25,11 @@
 #ifndef GCU_LOADER_H
 #define GCU_LOADER_H
 
+#include "structs.h"
 #include <list>
 #include <map>
 #include <string>
-#ifdef GOFFICE_HAS_GLOBAL_HEADER
-#   include <goffice/goffice.h>
-#else
-#   include <goffice/app/io-context.h>
-#endif
-#include <gcu/goffice-compat.h>
+#include <goffice/goffice.h>
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-output.h>
 
@@ -42,6 +38,7 @@ namespace gcu {
 
 class Document;
 class Loader;
+class Object;
 
 /*!\struct LoaderStruct gcu/loader.h
 	 Stores data for loaders. They are created when Loader::Init is called for
@@ -203,19 +200,20 @@ This function must be overloaded by a derived class able to read. Default
 implementation just return false.
 @return true on success, false otherwise.
 */
-	virtual bool Read (Document *doc, GsfInput *in, char const *mime_type, IOContext *io);
+	virtual ContentType Read (Document *doc, GsfInput *in, char const *mime_type, IOContext *io);
 /*!
-@param doc the gcu::Document being written.
+@param obj the gcu::Object being written.
 @param out a GsfOutput (see the libgsf documentation at
 http://library.gnome.org/devel/gsf/stable/gsf-Output-to-unstructured-files.html#GsfOutput).
 @param mime_type the mime type of the data.
 @param io a GOffice IOContext.
+@param type the type of the data to be saved (see gcu::ContentType).
 
 This function must be overloaded by a derived class able to write. Default
 implementation just return false.
 @return true on success, false otherwise.
 */
-	virtual bool Write (Document *doc, GsfOutput *out, char const *mime_type, IOContext *io);
+	virtual bool Write (Object *obj, GsfOutput *out, char const *mime_type, IOContext *io, ContentType type = ContentTypeMisc);
 
 protected:
 /*!
