@@ -4,7 +4,7 @@
  * GChemPaint residues plugin
  * residues-dlg.cc
  *
- * Copyright (C) 2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2007-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,7 +26,9 @@
 #include "residues-dlg.h"
 #include "pseudo-atom.h"
 #include <gcp/application.h>
+#include <gcp/bond.h>
 #include <gcp/document.h>
+#include <gcp/molecule.h>
 #include <gcp/residue.h>
 #include <gcp/theme.h>
 #include <gcp/view.h>
@@ -39,22 +41,22 @@ xmlDocPtr user_residues = NULL;
 set<xmlDocPtr> docs;
 
 
-static bool on_key_release (GtkWidget* widget, GdkEventKey* ev, gcpResiduesDlg *dlg)
+static bool on_key_release (G_GNUC_UNUSED GtkWidget* widget, GdkEventKey* ev, gcpResiduesDlg *dlg)
 {
 	return dlg->OnKeyRelease (ev);
 }
 
-static bool on_key_press (GtkWidget* widget, GdkEventKey* ev, gcpResiduesDlg *dlg)
+static bool on_key_press (G_GNUC_UNUSED GtkWidget* widget, GdkEventKey* ev, gcpResiduesDlg *dlg)
 {
 	return dlg->OnKeyPress (ev);
 }
 
-static void on_page (GtkNotebook *book, GtkNotebookPage *page, int num_page, gcpResiduesDlg *dlg)
+static void on_page (G_GNUC_UNUSED GtkNotebook *book, G_GNUC_UNUSED GtkNotebookPage *page, int num_page, gcpResiduesDlg *dlg)
 {
 	dlg->SetPage (num_page);
 }
 
-static void on_cur_changed (GtkComboBox *box, gcpResiduesDlg *dlg)
+static void on_cur_changed (G_GNUC_UNUSED GtkComboBox *box, gcpResiduesDlg *dlg)
 {
 	dlg->OnCurChanged ();
 }
@@ -69,23 +71,23 @@ static void on_delete (gcpResiduesDlg *dlg)
 	dlg->Remove ();
 }
 
-static void on_symbol_activate (GtkEntry *entry, gcpResiduesDlg *dlg)
+static void on_symbol_activate (G_GNUC_UNUSED GtkEntry *entry, gcpResiduesDlg *dlg)
 {
 	dlg->OnSymbolActivate ();
 }
 
-static bool on_symbol_focus_out (GtkEntry *entry, GdkEventFocus *event, gcpResiduesDlg *dlg)
+static bool on_symbol_focus_out (GtkEntry *entry, G_GNUC_UNUSED GdkEventFocus *event, gcpResiduesDlg *dlg)
 {
 	on_symbol_activate (entry, dlg);
 	return true;
 }
 
-static void on_name_activate (GtkEntry *entry, gcpResiduesDlg *dlg)
+static void on_name_activate (G_GNUC_UNUSED GtkEntry *entry, gcpResiduesDlg *dlg)
 {
 	dlg->OnNameActivate ();
 }
 
-static bool on_name_focus_out (GtkEntry *entry, GdkEventFocus *event, gcpResiduesDlg *dlg)
+static bool on_name_focus_out (GtkEntry *entry, G_GNUC_UNUSED GdkEventFocus *event, gcpResiduesDlg *dlg)
 {
 	on_name_activate (entry, dlg);
 	return true;
@@ -145,7 +147,7 @@ static void delete_symbol (GtkComboBox *box, char const *str)
 }
 
 gcpResiduesDlg::gcpResiduesDlg (gcp::Application *App):
-	Dialog (App, GLADEDIR"/residues.glade", "residues", App),
+	Dialog (App, UIDIR"/residues.ui", "residues", GETTEXT_PACKAGE, App),
 	gcp::Target (App)
 {
 	m_Document = new gcp::Document (App, true, NULL);
@@ -157,7 +159,7 @@ gcpResiduesDlg::gcpResiduesDlg (gcp::Application *App):
 	gtk_scrolled_window_add_with_viewport (scroll, w);
 	gtk_widget_set_size_request (GTK_WIDGET (scroll), 408, 308);
 	gtk_widget_show (GTK_WIDGET (scroll));
-	gtk_box_pack_start (GTK_BOX (glade_xml_get_widget (xml, "formula-box")), GTK_WIDGET (scroll), true, true, 0);
+	gtk_box_pack_start (GTK_BOX (GetWidget ("formula-box")), GTK_WIDGET (scroll), true, true, 0);
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 	App->SetActiveDocument (m_Document);
 	gcp::Theme *theme = m_Document->GetTheme ();
@@ -173,8 +175,8 @@ gcpResiduesDlg::gcpResiduesDlg (gcp::Application *App):
 	SetWindow (dialog);
 	g_signal_connect (dialog, "key-press-event", G_CALLBACK (on_key_press), this);
 	g_signal_connect (dialog, "key-release-event", G_CALLBACK (on_key_release), this);
-	g_signal_connect (glade_xml_get_widget (xml, "residue-book"), "switch-page", G_CALLBACK (on_page), this);
-	m_CurBox = GTK_COMBO_BOX (glade_xml_get_widget (xml, "cur-box"));
+	g_signal_connect (GetWidget ("residue-book"), "switch-page", G_CALLBACK (on_page), this);
+	m_CurBox = GTK_COMBO_BOX (GetWidget ("cur-box"));
 	ResidueIterator i;
 	string const *s = Residue::GetFirstResidueSymbol (i);
 	while (s) {
@@ -183,24 +185,24 @@ gcpResiduesDlg::gcpResiduesDlg (gcp::Application *App):
 	}
 	gtk_combo_box_set_active (m_CurBox, 0);
 	g_signal_connect (G_OBJECT (m_CurBox), "changed", G_CALLBACK (on_cur_changed), this);
-	m_SaveBtn = glade_xml_get_widget (xml, "save");
+	m_SaveBtn = GetWidget ("save");
 	g_signal_connect_swapped (G_OBJECT (m_SaveBtn), "clicked", G_CALLBACK (on_save), this);
-	m_DeleteBtn = glade_xml_get_widget (xml, "delete");
+	m_DeleteBtn = GetWidget ("delete");
 	g_signal_connect_swapped (G_OBJECT (m_DeleteBtn), "clicked", G_CALLBACK (on_delete), this);
-	m_SymbolEntry = GTK_ENTRY (glade_xml_get_widget (xml, "symbol-entry"));
+	m_SymbolEntry = GTK_ENTRY  (GetWidget ("symbol-entry"));
 	g_signal_connect (G_OBJECT (m_SymbolEntry), "activate", G_CALLBACK (on_symbol_activate), this);
 	g_signal_connect_after (G_OBJECT (m_SymbolEntry), "focus_out_event", G_CALLBACK (on_symbol_focus_out), this);
 	m_ValidSymbols = false;
-	m_NameEntry = GTK_ENTRY (glade_xml_get_widget (xml, "name-entry"));
+	m_NameEntry = GTK_ENTRY (GetWidget ("name-entry"));
 	g_signal_connect (G_OBJECT (m_NameEntry), "activate", G_CALLBACK (on_name_activate), this);
 	g_signal_connect_after (G_OBJECT (m_NameEntry), "focus_out_event", G_CALLBACK (on_name_focus_out), this);
 	m_ValidName = false;
-	m_GenericBtn = glade_xml_get_widget (xml, "generic-btn");
+	m_GenericBtn = GetWidget ("generic-btn");
 	g_signal_connect (m_GenericBtn, "toggled", G_CALLBACK (on_generic_toggled), this);
 	m_Generic = false;
 	m_Residue = NULL;
 	m_Page = 0;
-	g_signal_connect (glade_xml_get_widget (xml, "residue-book"), "change-current-page", G_CALLBACK (on_page_changed), this);
+	g_signal_connect (GetWidget ("residue-book"), "change-current-page", G_CALLBACK (on_page_changed), this);
 }
 
 gcpResiduesDlg::~gcpResiduesDlg ()
@@ -344,7 +346,7 @@ void gcpResiduesDlg::Remove ()
 
 bool gcpResiduesDlg::Close ()
 {
-	Destroy ();
+	Dialog::Destroy ();
 	return true;
 }
 
@@ -421,7 +423,7 @@ void gcpResiduesDlg::OnCurChanged ()
 		gtk_widget_set_sensitive (reinterpret_cast<GtkWidget*> (m_NameEntry), true);
 		gtk_widget_set_sensitive (reinterpret_cast<GtkWidget*> (m_SymbolEntry), true);
 		gtk_widget_set_sensitive (m_SaveBtn, true);
-		gtk_widget_set_sensitive (m_DeleteBtn, true);
+		gtk_widget_set_sensitive (m_DeleteBtn, m_Residue->GetRefs () == 0);
 		gtk_widget_set_sensitive (m_GenericBtn, true);
 		m_Document->SetEditable (true);
 	}
@@ -436,12 +438,14 @@ void gcpResiduesDlg::OnCurChanged ()
 	gtk_entry_set_text (m_SymbolEntry, sy.c_str ());	
 	m_Document->Clear ();
 	m_Document->LoadObjects (m_Residue->GetMolNode ());
-	double r =  m_Document->GetMedianBondLength () / m_Document->GetTheme ()->GetBondLength ();
+	double r =  m_Document->GetTheme ()->GetBondLength () / m_Document->GetMedianBondLength ();
 	if (fabs (r - 1.) > .0001) { 
 		Matrix2D m (r, 0., 0., r);
 		m_Document->Transform2D (m, 0., 0.);
 		m_Document->GetView ()->Update (m_Document);
+		m_Document->GetView ()->EnsureSize ();
 	}
+	m_Document->GetView ()->EnsureSize ();
 	m_Atom = dynamic_cast <gcpPseudoAtom *> (m_Document->GetDescendant ("a1"));
 	map<gcu::Atom*, gcu::Bond*>::iterator j;
 	m_Atom->GetFirstBond (j);
@@ -465,3 +469,14 @@ void gcpResiduesDlg::OnNameActivate ()
 	m_ValidName = strlen (text) > 0;
 	gtk_widget_set_sensitive (m_SaveBtn, m_ValidName && m_ValidSymbols);
 }
+
+void gcpResiduesDlg::OnNewResidue (gcp::Residue *res)
+{
+	if (res) {
+		map<string, bool> const &symbols = res->GetSymbols ();
+		map<string, bool>::const_iterator i = symbols.begin (), end = symbols.end ();
+		for (i = symbols.begin (); i != end; i++)
+			gtk_combo_box_append_text (m_CurBox, (*i).first.c_str ());
+	} else if (m_Residue && !m_Residue->GetReadOnly ())
+		gtk_widget_set_sensitive (m_DeleteBtn, m_Residue->GetRefs () == 0);
+}
