@@ -25,13 +25,6 @@
 #include "config.h"
 #include "standaloneapp.h"
 #include <gcu/loader.h>
-#ifndef GOFFICE_HAS_GLOBAL_HEADER
-#   include <goffice/utils/go-file.h>
-#   include <gtk/gtk.h>
-#endif
-#ifdef GOFFICE_IS_0_6
-#	include <libgnomevfs/gnome-vfs-init.h>
-#endif
 #include <glib/gi18n-lib.h>
 #include <cstring>
 
@@ -66,9 +59,6 @@ int main(int argc, char *argv[])
 	textdomain (GETTEXT_PACKAGE);
 
 	gtk_init (&argc, &argv);
-#ifdef GOFFICE_IS_0_6
-	gnome_vfs_init ();
-#endif
 	App = new gcpStandaloneApp();
 	if (argc > 1 && argv[1][0] == '-') {
 		context = g_option_context_new (_(" [file...]"));
